import xml.etree.ElementTree as ET
import enum

########################################################################################################################
#
#  class ANA6705Reg, ANA6706Reg
#  class TRegister
#
########################################################################################################################

class ANA6705 :
    NOP=0x00
    SWRESET=0x01
    RDCMODE=0x03
    RDDIDIF=0x04
    RDNUMPE=0x05
    RDRED=0x06
    RDGREEN=0x07
    RDBLUE=0x08
    RDDPM=0x0A
    RDDMADCTL=0x0B
    RDDIM=0x0D
    RDDSM=0x0E
    RDDSDR=0x0F
    SLPIN=0x10
    SLPOUT=0x11
    NORON=0x13
    INVOFF=0x20
    INVON=0x21
    ALLPOFF=0x22
    ALLPON=0x23
    DISPOFF=0x28
    DISPON=0x29
    CASET=0x2A
    PASET=0x2B
    RAMWR=0x2C
    RAMRD=0x2E
    TEOFF=0x34
    TEON=0x35
    MADCTL=0x36
    IDMOFF=0x38
    IDMON=0x39
    RAMWRC=0x3C
    RAMRDC=0x3E
    TESCL=0x44
    RDSCL=0x45
    WRDISBV=0x51
    RDDISBV=0x52
    WRCTRLD=0x53
    RDCTRLD=0x54
    WRCABC=0x55
    RDCABC=0x56
    WRVRRCTRL=0x60
    RDVRRCTRL=0x61
    WRIP00=0x81
    RDIP00=0x82
    WRIP01=0x83
    RDIP01=0x84
    WRIP02=0x85
    RDIP02=0x86
    WRIP03=0x87
    RDIP03=0x88
    WRIP04=0x89
    RDIP04=0x8A
    WRIP07=0x8B
    RDIP07=0x8C
    WRIP08=0x8E
    RDIP08=0x8F
    WRIP17=0x90
    RDIP17=0x91
    WRCMOD=0x9D
    WRPPS=0x9E
    PASSWD0=0x9F
    RDDDBS=0xA1
    RDPPSS=0xA2
    RDDDBC=0xA8
    RDPPSC=0xA9
    RDFCS=0xAA
    RDCCS=0xAF
    RDID1=0xDA
    RDID2=0xDB
    RDID3=0xDC
    G_PARA=0xB0
    IP00CTL=0xB1
    IP01CTL=0xB2
    IP02CTL=0xB3
    IP03CTL=0xB4
    IP04CTL=0xB5
    IP05CTL=0xB6
    IP06CTL_0=0xB7
    IP06CTL_1=0xB8
    IP06CTL_2=0xB9
    IP06CTL_3=0xBA
    IP06CTL_4=0xBB
    IP06CTL_5=0xBC
    IP07CTL=0xBD
    IP08CTL_0=0xBE
    IP08CTL_1=0xBF
    IP08CTL_2=0xC0
    IP08CTL_3=0xC1
    IP08CTL_4=0xC2
    IP09CTL=0xC3
    IP10CTL=0xC4
    IP11CTL_0=0xC5
    IP11CTL_1=0xC6
    IP11CTL_2=0xC7
    IP12CTL=0xC8
    IP13CTL=0xC9
    IP14CTL=0xCA
    IP15CTL=0xCB
    IP16CTL=0xCC
    IP17CTL=0xCD
    IP18CTL_0=0xCE
    IP18CTL_1=0xCF
    IP19CTL=0xD0
    IPTOPCTL=0xD1
    GPOCTL=0xD2
    OTPCTRL=0xD3
    OTPCHECK=0xD4
    MCDCTL=0xD5
    CHIP_ID=0xD6
    TECTRL=0xD7
    NVMRD=0xD8
    DSTB=0xD9
    MPSCTL=0xDD
    SPICTL=0xDE
    SPIDATA=0xDF
    PCDCTL=0xE0
    RPCDCTL=0xE1
    SCALER=0xE2
    MIPI_LANE_SEL=0xE3
    LFDCTL=0xE4
    MIPI0=0xE5
    IP22CTL=0xE6
    CCDCTL=0xE7
    IP26CTL=0xE8
    IP27CTL=0xE9
    MIPI5=0xEA
    IP28CTL=0xEB
    LABA=0xEC
    ESDFG=0xED
    ESDERR=0xEE
    DBIST=0xEF
    PASSWD1=0xF0
    PASSWD2=0xF1
    DISPCTL=0xF2
    MANPWRSEQ=0xF3
    PWRCTL=0xF4
    T2MCTL=0xF5
    SRCCTL=0xF6
    PANELUPDATE=0xF7
    ANAIPCTL=0xF8
    MIPICTL=0xF9
    RDOPTREV=0xFA
    MISCCTL=0xFB
    PASSWD3=0xFC
    IP29CTL=0xFD
    IP33CTL=0xFE

class ANA6706 :
    NOP=0x00
    SWRESET=0x01
    RDCMODE=0x03
    RDDIDIF=0x04
    RDNUMPE=0x05
    RDRED_DSC=0x06
    RDGREEN_DSC=0x07
    RDBLUE_DSC=0x08
    RDDPM=0x0A
    RDDMADCTL=0x0B
    RDDIM=0x0D
    RDDSM=0x0E
    RDDSDR=0x0F
    SLPIN=0x10
    SLPOUT=0x11
    PTLON=0x12
    NORON=0x13
    INVOFF=0x20
    INVON=0x21
    ALLPOFF=0x22
    ALLPON=0x23
    DISPOFF=0x28
    DISPON=0x29
    CASET=0x2A
    PASET=0x2B
    RAMWR=0x2C
    RAMRD=0x2E
    PTLAR=0x30
    TEOFF=0x34
    TEON=0x35
    MADCTL=0x36
    IDMOFF=0x38
    IDMON=0x39
    RAMWRC=0x3C
    RAMRDC=0x3E
    TESCL=0x44
    RDSCL=0x45
    WRDISBV=0x51
    RDDISBV=0x52
    WRCTRLD=0x53
    RDCTRLD=0x54
    WRCABC=0x55
    RDCABC=0x56
    WRVRRCTRL=0x60
    RDVRRCTRL=0x61
    WRIP00=0x82
    RDIP00=0x83
    WRIP01=0x84
    RDIP01=0x85
    WRIP02=0x86
    RDIP02=0x87
    WRIP03=0x88
    RDIP03=0x89
    WRIP04=0x8A
    RDIP04=0x8B
    WRIP07=0x8C
    RDIP07=0x8D
    WRIP08=0x8E
    RDIP08=0x8F
    WRCMOD=0x9D
    WRPPS=0x9E
    PASSWD0=0x9F
    RDDDBS=0xA1
    RDPPSS=0xA2
    WRIP37=0xA4
    RDIP37=0xA5
    HAP_W=0xA6
    HAP_R=0xA7
    RDDDBC=0xA8
    RDPPSC=0xA9
    RDFCS=0xAA
    RDCCS=0xAF
    RDID1=0xDA
    RDID2=0xDB
    RDID3=0xDC
    G_PARA=0xB0
    IP00=0xB8
    IP01=0xBC
    IP02=0xBE
    IP03_IP04=0xC2
    R=0x90
    BCCTL=0xB1
    OLEDREAD=0xB2
    HBMCTL=0xB3
    ACLCTL=0xB4
    MPSCTL=0xB5
    RMPSCTL=0xB6
    HLPMCTL=0xBB
    GAMCTL1=0xC7
    GAMCTL2=0xC8
    GAMCTL3=0xC9
    GAMCTL4=0xCA
    GAMMODE1=0xCA
    IP07_IP09=0x91
    IP08A=0xC3
    IP08B=0xC4
    IP08C=0xC5
    IP08D=0xC6
    IP08E=0xCE
    IP08F=0xCF
    IP10_IP12=0x92
    IP11A=0x93
    IP11B=0x94
    IP11C=0x95
    IP11D=0xA0
    IP11E=0xA3
    IP11F=0xAB
    IP11G=0xAC
    IP11F=0xAB
    IP11H=0xAD
    IP13A=0x6D
    IP13B=0x6F
    IP14=0x96
    IP15A=0x97
    IP15B=0x98
    IP15C=0x9C
    IP16=0x99
    IP18A=0x9A
    IP18B=0x9B
    IP19=0x63
    IP20_IP29=0xD2
    IP22=0xD3
    IP23=0xD4
    IP25=0x64
    IP27=0x65
    IP33_37=0xD5
    IP36A=0xD7
    IP36B=0xE0
    IP36C=0xE2
    IP36D=0xE4
    IP36E=0xE7
    IP36F=0xE8
    IP36G=0xEA
    IPTOP=0x6A
    GPO_A=0xCB
    GPO_B=0xB7
    TECTRL=0xB9
    LFDCTL=0xBD
    DBIST=0xBF
    SPICTL=0xC0
    SPIDATA=0xC1
    PCD_CCDCTL=0xCC
    MCDCTL=0xCD
    OTPCTRL=0xD0
    OTPCHECK=0xD1
    CHIP_ID=0xD6
    NVMRD=0xD8
    DSTB=0xD9
    MIPI0=0xE5
    MIPICTL=0xE6
    MIPI5=0xE9
    ESDFG=0xED
    ESDERR=0xEE
    PWRSEQ=0xEF
    PASSWD1=0xF0
    PASSWD2=0xF1
    DISPCTL=0xF2
    PWRCTL=0xF4
    T2MCTL=0xF5
    SRCCTL=0xF6
    PANELUPDATE=0xF7
    ANAIPCTL=0xF8
    FAILSAFE=0xF9
    RDOPTREV=0xFA
    MISCCTL=0xFB


class Xml :

    class NodeName(enum.Enum) :
        Register="Register"
        Property="Property"
  
    def __init__(this, chipType):
        if chipType == TChip.Type.ANA6705 :
            tree=ET.parse("ANA6705_Register.xml")
        elif chipType == TChip.Type.ANA6706 :
            tree=ET.parse("ANA6706_Register.xml")
        else :
            raise ValueError("[TRegister::TRegister] Not Suported ChipType,  Check TChip.Type")
        this.__Root = tree.getroot()
        this.__Dictionary=dict()
        
        for registerNode in this.__Root :
            if registerNode.tag == this.NodeName.Register.value :
                propNode = registerNode.find('Property')
                this.__Dictionary[propNode.text] = registerNode

    def print(this) :
        for registerNode in this.__Dictionary.values() :
            regAddr = registerNode.get('ID')
            regAddr = int(regAddr, 16)
            propNode = registerNode.find('Property')
            print(propNode.text + "  " + '{:#x}'.format(regAddr) )

    def PrintAllRegName(this) :
        for regNode in this.__Root :
            if regNode.tag == this.NodeName.Register.value :
                regName = regNode.find('Property').text
                regAddr = int(regNode.get('ID'), 16)
                print(regName+'='+'0x%02X' % regAddr )

    def GetAddr(this, regName) :
         nid=regName.upper()
         regNode = this.__Dictionary[nid]
         regAddr = int(regNode.get('ID'), 16)
         return regAddr

    def __getitem__(this, regName) :
        return this.GetAddr(regName)   


class XmlANA6705(Xml) :    
    def __init__(this) :
        Xml.__init__(this, TChip.Type.ANA6705)

class XmlANA6706(Xml) :    
    def __init__(this) :
        Xml.__init__(this, TChip.Type.ANA6706)




from Anapass import TModule
from Anapass import FlashMemory
from Anapass.AppShell import AppClass
from Anapass import Util
import easygui 
import time
import threading

def ReadStatus(app, tupleArg) :
    flash = FlashMemory.CreateInstance(app.GetDevice(), app.GetChipType(), FlashMemory.Type.MipiToSpi)
    status1 = flash.ReadStatus1()
    status2 = flash.ReadStatus2()
    print("status1= ", "0x%02X"%status1)
    print("status2= ", "0x%02X"%status2)

def ReadStatus1(app, tupleArg) :
    flash = FlashMemory.CreateInstance(app.GetDevice(), app.GetChipType(), FlashMemory.Type.MipiToSpi)
    status1 = flash.ReadStatus1()
    print("status1= ", "0x%02X"%status1)
    
def ReadStatus2(app, tupleArg) :
    flash = FlashMemory.CreateInstance(app.GetDevice(), app.GetChipType(), FlashMemory.Type.MipiToSpi)
    status2 = flash.ReadStatus2()
    print("status2= ", "0x%02X"%status2)

def WriteStatus1(app, tupleArg) :
    try :
        writeStatus = int(tupleArg[0], 16)
    except IndexError :
        print("[WriteStatus1] tupleArg[0] IndexError, input StatusNumber(Hex)")
    except ValueError :
        print("[WriteStatus1] tupleArg[0] ValueError,  Value is not convertable to Hex-Integer")
    else :
        print("[WriteStatus1] writeStaus= 0x%02X" % writeStatus)
        flash = FlashMemory.CreateInstance(app.GetDevice(), app.GetChipType(), FlashMemory.Type.MipiToSpi)
        flash.WriteStatus1(writeStatus)
        vefifyStatus = flash.ReadStatus1()
        print("[WriteStatus1] vefifyStatus= 0x%02X" % vefifyStatus)
        
def WriteStatus2(app, tupleArg) :
    try :
        writeStatus = int(tupleArg[0], 16)
    except IndexError :
        print("[WriteStatus2] tupleArg[0] IndexError, input StatusNumber(Hex)")
    except ValueError :
        print("[WriteStatus2] tupleArg[0] ValueError,  Value is not convertable to Hex-Integer")
    else :
        print("[WriteStatus2] writeStaus= 0x%02X" % writeStatus)
        flash = FlashMemory.CreateInstance(app.GetDevice(), app.GetChipType(), FlashMemory.Type.MipiToSpi)
        flash.WriteStatus2(writeStatus)
        vefifyStatus = flash.ReadStatus2()
        print("[WriteStatus2] vefifyStatus= 0x%02X" % vefifyStatus)

def Lock(app, tupleArg) :
    flash = FlashMemory.CreateInstance(app.GetDevice(), app.GetChipType(), FlashMemory.Type.MipiToSpi)
    flash.Lock();
        
def Unlock(app, tupleArg) :
    flash = FlashMemory.CreateInstance(app.GetDevice(), app.GetChipType(), FlashMemory.Type.MipiToSpi)
    flash.Unlock();

def IsLock(app, tupleArg) :
    flash = FlashMemory.CreateInstance(app.GetDevice(), app.GetChipType(), FlashMemory.Type.MipiToSpi)
    isLock = flash.IsLock()
    if(isLock) :
        print("TFlash is Lock")
    else :
        print("TFlash is Unlock")

def EraseSector(app, tupleArg) :
    try :
        sectorAddr = int(tupleArg[0], 16)
    except IndexError :
        print("IndexError, input sectorAddress(Hex) ex) tflash eraseSector 0x01 ")
    except ValueError :
        print("ValueError,  Value is not convertable to Hex-Integer ex) tflash eraseSector 0x01")
    else :
        flash = FlashMemory.CreateInstance(app.GetDevice(), app.GetChipType(), FlashMemory.Type.MipiToSpi)
        sectorMaxCount = flash.GetSectorMaxCount()
        print("sectorAddr Range : 0x0000 ~ 0x%04X"%(sectorMaxCount-1))
        if sectorAddr < sectorMaxCount:
            bRet = flash.EraseSector(sectorAddr)
            if bRet :
                print("Erase sector(0x%04X) OK"%sectorAddr)
            else :
                print("Erase sector(0x%04X) Fail"%sectorAddr)
        else :
            print("sectorAddr is invalid")

def __ReadService(flash, memAddr, readDataByteSize, isFileDump, workAgent) :
    print("__ReadService is starting..")
    outData=[0 for _ in range(readDataByteSize)]  #읽을 Register 값 개수만큼 리스트 할당 
    workAgent.Start()
    flash.Read(memAddr, readDataByteSize, outData, workAgent)
    if isFileDump :
        filePath = easygui.filesavebox()
        fp = open(filePath, "wb")
        fp.write(bytes(outData))
        fp.close()
    else :
        Util.Print.ListHex(memAddr, outData)
    workAgent.Stop()
    print("__ReadService is finished")
    
def __Read(app, tupleArg, isFileDump) :

    try :
        memAddr = int(tupleArg[0], 16)
    except IndexError :
        print("IndexError, check readAddr(hex)")
        print("   Usage) tflash read [Addr(hex)] [readByteSize(dec)]")
        return 
    except ValueError :
        print("ValueError,  The address of memory is not convertable to Hex-Integer")
        print("   Usage) tflash read [Addr(hex)] [readByteSize(dec)]")
        return 
    
    try :
        readDataByteSize = int(tupleArg[1], 10)
    except IndexError :
        print("IndexError, check readByteSize(dec)")
        print("   Usage) tflash read [Addr(hex)] [readByteSize(dec)]")
        return 
    except ValueError :
        print("ValueError,  The size is not convertable to Dec-Integer")
        print("   Usage) tflash read [Addr(hex)] [readByteSize(dec)]")
        return 

    flash = FlashMemory.CreateInstance(app.GetDevice(), app.GetChipType(), FlashMemory.Type.MipiToSpi)
    if memAddr+readDataByteSize >= flash.GetCapacityByteSize() :
        print("Flash Memroy Range :  0x00000000 ~ 0x%08X" % (flash.GetCapacityByteSize()-1))
        print("Out of Range :  memAddr=0x%08x Size=%d " % memAddr, readDataByteSize)
        return 

    workAgent = Util.WorkAgent()
    workAgent.SetMaxValue(readDataByteSize)
    thread = threading.Thread(target=__ReadService, args=(flash,  memAddr, readDataByteSize, isFileDump, workAgent))
    thread.start()
    while True :
        maxV = workAgent.GetMaxValue()
        curV = workAgent.GetCurValue()
        print("%d"%curV, "/",  "%d"%maxV, end='\r')
        if workAgent.IsStop() :
            print("Read Work is stopped")
            break
        time.sleep(0.3)


def Read(app, tupleArg) :
    __Read(app, tupleArg, False)
    

def Dump(app, tupleArg) :
    __Read(app, tupleArg, True)

def __WriteService(flash, memAddr, writeData, workAgent) :
    print("__WriteService is starting..")
    workAgent.Start()
    flash.Write(memAddr, len(writeData), writeData, workAgent)
    workAgent.Stop()
    print("__WriteService is finished")


def FileWrite(app, tupleArg) :    

    try :
        memAddr = int(tupleArg[0], 16)
    except IndexError :
        print("IndexError, check readAddr(hex)")
        print("   Usage) tflash FileWrite [Addr(hex)] ")
        return 
    except ValueError :
        print("ValueError,  The address of memory is not convertable to Hex-Integer")
        print("   Usage) tflash FileWrite [Addr(hex)] ")
        return 

    filePath = easygui.fileopenbox()
    fp = open(filePath, "rb")
    fileData = fp.read()
    fileSize = len(fileData)
    fp.close()

    flash = FlashMemory.CreateInstance(app.GetDevice(), app.GetChipType(), FlashMemory.Type.MipiToSpi)

    
    print("FileName : " + filePath)
    print("FileSize : ", fileSize)
    print("Flash Memory Addres : ", "0x%08X"%memAddr )

    workAgent = Util.WorkAgent()
    workAgent.SetMaxValue(len(fileData))
    thread = threading.Thread(target=__WriteService, args=(flash,  memAddr, fileData, workAgent))
    thread.start()
    while True :
        maxV = workAgent.GetMaxValue()
        curV = workAgent.GetCurValue()
        print("%d"%curV, "/",  "%d"%maxV, end='\r')
        if workAgent.IsStop() :
            print("Write Work is stopped")
            break
        time.sleep(0.3)
    
    #flash.Write(memAddr, len(fileData), fileData)
    #fp1 = open("d:\\work\\test.jpg", "wb")
    #fp1.write(bytes(fileData))
    #fp1.close()



shellCmd = AppClass.ShellCommand()
shellCmd.Add("ReadStatus", ReadStatus, "ReadStatus")
shellCmd.Add("ReadStatus1", ReadStatus1, "ReadStatus1")
shellCmd.Add("ReadStatus2", ReadStatus2, "ReadStatus2")
shellCmd.Add("WriteStatus1", WriteStatus1, "WriteStatus1")
shellCmd.Add("WriteStatus2", WriteStatus2, "WriteStatus2")
shellCmd.Add("Lock", Lock, "Lock")
shellCmd.Add("Unlock", Unlock, "Unlock")
shellCmd.Add("IsLock", IsLock, "IsLock")
shellCmd.Add("EraseSector", EraseSector, "EraseSector")
shellCmd.Add("Read", Read, "Read")
shellCmd.Add("Dump", Dump, "Dump")
shellCmd.Add("FileWrite", FileWrite, "FileWrite")

def Main(app, tupleArg) :
    if len(tupleArg)==0 or tupleArg[0] == "help" : 
        shellCmd.PrintHelp()
    else :
        isRunCmd = False
        cmd = tupleArg[0]
        tupleArg = tupleArg[1:]
        for shellItem in shellCmd :
                if shellItem.Run(app, cmd, tupleArg) == True :
                    isRunCmd = True
                    break
        if isRunCmd == False :
            print("[TFlash] Can't find sub command")
    



import enum
import struct

class Vlin1 :

    class Type(enum.IntEnum) :
        VLIN1=0
        VBAT=1
        ELVDD=2

    def __init__(this) : 
        this.ChannelCount=3
        this.Voltage=[-1 for _ in range(this.ChannelCount)]
        this.Current=[-1 for _ in range(this.ChannelCount)]

class Vci :
    class Type(enum.IntEnum) :
        VCI=0
        VDDR=1
        VDDI=2
    def __init__(this) : 
        this.ChannelCount=3
        this.Voltage=[-1 for _ in range(this.ChannelCount)]
        this.Current=[-1 for _ in range(this.ChannelCount)]

class Sdout :
    def __init__(this, channelCount) : 
        this.Voltage=[-1 for _ in range(channelCount)]

class Ldo :
    def __init__(this, channelCount) : 
        this.Voltage=[-1 for _ in range(channelCount)]

class Reg :
    def __init__(this, channelCount) : 
        this.Voltage=[-1 for _ in range(channelCount)]

class Measure :
    def __init__(this, soutChannelCount, ldoChannelCount, regChannelCount): 
        
        if soutChannelCount < 0 :  soutChannelCount = 0
        if ldoChannelCount < 0 :  ldoChannelCount = 0
        if regChannelCount < 0 :  regChannelCount = 0

        this.__SoutChannelCount = soutChannelCount
        this.__LdoChannelCount = ldoChannelCount
        this.__RegChannelCount = regChannelCount
        
        this.vlin1 = Vlin1()
        this.vci = Vci()
        this.sdout = Sdout(this.__SoutChannelCount)
        this.ldo = Ldo(this.__LdoChannelCount)
        this.reg = Reg(this.__RegChannelCount)
        
    
    def GetStructData(this) :
        this.__StructFmt = 'iiiiiiii'    #   Vlin1Adc voltage/current
        this.__StructFmt += 'iiiiiiii'    #   VciAdc voltage/current
        for i in range(16) :   this.__StructFmt += 'i'    # Sdout
        for i in range(16) :   this.__StructFmt += 'i'    # Ldo
        for i in range(16) :   this.__StructFmt += 'i'    # Reg
        this.__StructMeasureData = struct.pack(this.__StructFmt, 
                           0, 0, 0, 0, 0, 0, 0, 0,
                           0, 0, 0, 0, 0, 0, 0, 0,
                           0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                           0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                           0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                           )

        return this.__StructMeasureData

    def ParseStructData(this) :
 
        resMea= struct.unpack(this.__StructFmt, this.__StructMeasureData)

        resIdx = 0
        
        #Vlin1Adc
        for i in range(3) :  this.vlin1.Voltage[i] = resMea[resIdx+i]  #Vlin1 Voltage
        resIdx+=4
        for i in range(3) :  this.vlin1.Current[i] = resMea[resIdx+i]  #Vlin1 Current
        resIdx+=4
        
        #VciADC
        for i in range(3) :  this.vci.Voltage[i] = resMea[resIdx+i]  #Vlin1 Voltage
        resIdx+=4
        for i in range(3) :  this.vci.Current[i] = resMea[resIdx+i]  #Vlin1 Current
        resIdx+=4
        
        #Sdout
        for i in range(this.__SoutChannelCount) :  this.sdout.Voltage[i] = resMea[resIdx+i]  
        resIdx+=16

        #Ldo
        for i in range(this.__LdoChannelCount) :  this.ldo.Voltage[i] = resMea[resIdx+i]  
        resIdx+=16

        #Reg
        for i in range(this.__RegChannelCount) :  this.reg.Voltage[i] = resMea[resIdx+i]  
        resIdx+=16

    def ParseListData(this, resList) :
 
        resIdx = 0


        
        #Vlin1Adc
        for i in range(3) :  this.vlin1.Voltage[i] = resList[resIdx+i]  #Vlin1 Voltage
        resIdx+=4
        for i in range(3) :  this.vlin1.Current[i] = resList[resIdx+i]  #Vlin1 Current
        resIdx+=4
        
        #VciADC
        for i in range(3) :  this.vci.Voltage[i] = resList[resIdx+i]  #Vlin1 Voltage
        resIdx+=4
        for i in range(3) :  this.vci.Current[i] = resList[resIdx+i]  #Vlin1 Current
        resIdx+=4
        
        #Sdout
        for i in range(this.__SoutChannelCount) :  this.sdout.Voltage[i] = resList[resIdx+i]  
        resIdx+=16

        #Ldo
        for i in range(this.__LdoChannelCount) :  this.ldo.Voltage[i] = resList[resIdx+i]  
        resIdx+=16

        #Reg
        for i in range(this.__RegChannelCount) :  this.reg.Voltage[i] = resList[resIdx+i]  
        resIdx+=16


    def Print(this) :        
        print("Vlin1Adc Voltage : ")
        print("\tVLINT1 : %d" % this.vlin1.Voltage[Vlin1.Type.VLIN1])
        print("\tVBAT   : %d" % this.vlin1.Voltage[Vlin1.Type.VBAT])
        print("\tELVDD  : %d" % this.vlin1.Voltage[Vlin1.Type.ELVDD])

        print("Vlin1Adc Current : ")
        print("\tVLINT1 : %d" % this.vlin1.Current[Vlin1.Type.VLIN1])
        print("\tVBAT   : %d" % this.vlin1.Current[Vlin1.Type.VBAT])
        print("\tELVDD  : %d" % this.vlin1.Current[Vlin1.Type.ELVDD])

        print("VciAdc Voltage : ")
        print("\tVCI  : %d" % this.vci.Voltage[Vci.Type.VCI])
        print("\tVDDR : %d" % this.vci.Voltage[Vci.Type.VDDR])
        print("\tVDDI : %d" % this.vci.Voltage[Vci.Type.VDDI])

        print("VciAdc Current : ")
        print("\tVCI  : %d" % this.vci.Current[Vci.Type.VCI])
        print("\tVDDR : %d" % this.vci.Current[Vci.Type.VDDR])
        print("\tVDDI : %d" % this.vci.Current[Vci.Type.VDDI])

        print("SdoutAdc Voltage : ", this.sdout.Voltage)
        print("ldoAdc Voltage : ", this.ldo.Voltage)
        print("RegAdc Voltage : ", this.reg.Voltage)



import sys
import platform
from ctypes import *
from ctypes.wintypes import *

class Api :

    def __init__(this) :
        #print(platform.architecture())
        is64bit = sys.maxsize > 2**32
        if is64bit :
            dllName="AnapassTModule.dll"
        else :
            dllName="AnapassTModule32.dll"

        try:
            #print("Load DLL from CurrentFolder " + "(./" + dllName + ")")
            Load_DLL = WinDLL('../TEDPython/' + dllName)
        except OSError as err:
            #print("OS error: {0}".format(err))
            #print("Load DLL from PackageFolder " + "(" + dllName + ")")
            Load_DLL = WinDLL(dllName)
    
        #TDEVICE_API TED_RESULT TSys_WinInit();
        this.TSys_WinInit = Load_DLL['TSys_WinInit']
        this.TSys_WinInit.restype=c_int;

        #TDEVICE_API TDEVICE_HDL TDeviceCreate(const TED_CHAR* fileName);
        this.TDeviceCreate=Load_DLL['TDeviceCreate']
        this.TDeviceCreate.restype=c_void_p
        this.TDeviceCreate.argtypes=[c_int]

        #TDEVICE_API TED_RESULT TDeviceDestroy(TDEVICE_HDL hdl);
        this.TDeviceDestroy=Load_DLL['TDeviceDestroy']
        this.TDeviceDestroy.restype=c_int;
        this.TDeviceDestroy.argtypes=[c_void_p]

        #TDEVICE_API TED_BOOL TDeviceSetServerIPAddr(TDEVICE_HDL hdl, const char* serverIPAddr); 
        this.TDeviceSysSetServerIPAddr = Load_DLL['TDeviceSysSetServerIPAddr']
        this.TDeviceSysSetServerIPAddr.restype=c_int;
        this.TDeviceSysSetServerIPAddr.argtypes=[c_void_p, c_char_p]

        #TDEVICE_API TED_BOOL TDeviceSetBoardID(TDEVICE_HDL hdl, int boardID); 
        this.TDeviceSysSetBoardID = Load_DLL['TDeviceSysSetBoardID']
        this.TDeviceSysSetBoardID.restype=c_int;
        this.TDeviceSysSetBoardID.argtypes=[c_void_p, c_int]

        #TDEVICE_API TED_BOOL TDeviceSysDelay(TDEVICE_HDL hdl, int delay /*unit : milesec*/);
        this.TDeviceSysDelay = Load_DLL['TDeviceSysDelay']
        this.TDeviceSysDelay.restype=c_int;
        this.TDeviceSysDelay.argtypes=[c_void_p, c_int]

        #TDEVICE_API int TDeviceSysGetDutCount(void* hdl);
        this.TDeviceSysGetDutCount = Load_DLL['TDeviceSysGetDutCount']
        this.TDeviceSysGetDutCount.restype=c_int;
        this.TDeviceSysGetDutCount.argtypes=[c_void_p]

        #TDEVICE_API int TDeviceSysGetDutIndexAllDeviceValue(void* hdl);
        this.TDeviceSysGetDutIndexAllDeviceValue = Load_DLL['TDeviceSysGetDutIndexAllDeviceValue']
        this.TDeviceSysGetDutIndexAllDeviceValue.restype=c_int;
        this.TDeviceSysGetDutIndexAllDeviceValue.argtypes=[c_void_p]

        #TDEVICE_API long long TDeviceSysGetCurUtcTime(void* hdl);
        this.TDeviceSysGetCurUtcTime = Load_DLL['TDeviceSysGetCurUtcTime']
        this.TDeviceSysGetCurUtcTime.restype=c_int;
        this.TDeviceSysGetCurUtcTime.argtypes=[c_void_p]

        #TDEVICE_API long long TDeviceSysGetTickCount64(void* hdl);
        this.TDeviceSysGetTickCount64 = Load_DLL['TDeviceSysGetTickCount64']
        this.TDeviceSysGetTickCount64.restype=c_int;
        this.TDeviceSysGetTickCount64.argtypes=[c_void_p]

        #TDEVICE_API long long TDeviceSysGetUtcTimeKST(void* hdl, int year, int month, int day, int hour, int min, int sec);
        this.TDeviceSysGetUtcTimeKST = Load_DLL['TDeviceSysGetUtcTimeKST']
        this.TDeviceSysGetUtcTimeKST.restype=c_int;
        this.TDeviceSysGetUtcTimeKST.argtypes=[c_void_p, c_int, c_int, c_int, c_int, c_int, c_int]

        #TDEVICE_API long long TDeviceSysGetErrFlag(void* hdl);
        this.TDeviceSysGetErrFlag = Load_DLL['TDeviceSysGetErrFlag']
        this.TDeviceSysGetErrFlag.restype=c_int;
        this.TDeviceSysGetErrFlag.argtypes=[c_void_p]

        #TDEVICE_API TED_BOOL TDeviceSysMipiLock(void* hdl);
        this.TDeviceSysMipiLock = Load_DLL['TDeviceSysMipiLock']
        this.TDeviceSysMipiLock.restype=c_int;
        this.TDeviceSysMipiLock.argtypes=[c_void_p]

        #TDEVICE_API TED_BOOL TDeviceSysMipiUnlock(void* hdl);
        this.TDeviceSysMipiUnlock = Load_DLL['TDeviceSysMipiUnlock']
        this.TDeviceSysMipiUnlock.restype=c_int;
        this.TDeviceSysMipiUnlock.argtypes=[c_void_p]

        #TDEVICE_API TED_BOOL TDeviceSysMipiIsLock(void* hdl);
        this.TDeviceSysMipiIsLock = Load_DLL['TDeviceSysMipiIsLock']
        this.TDeviceSysMipiIsLock.restype=c_int;
        this.TDeviceSysMipiIsLock.argtypes=[c_void_p]

                
        #TDEVICE_API TED_RESULT TDeviceConnect(TDEVICE_HDL hdl); 
        this.TDeviceConnect = Load_DLL['TDeviceConnect']
        this.TDeviceConnect.restype=c_int;
        this.TDeviceConnect.argtypes=[c_void_p]

        #TDEVICE_API TED_RESULT TDeviceDisconnect(TDEVICE_HDL hdl);
        this.TDeviceDisconnect = Load_DLL['TDeviceDisconnect']
        this.TDeviceDisconnect.restype=c_int;
        this.TDeviceDisconnect.argtypes=[c_void_p]

        #TDEVICE_API TED_RESULT TDeviceIsConnect(TDEVICE_HDL hdl);
        this.TDeviceIsConnect = Load_DLL['TDeviceIsConnect']
        this.TDeviceIsConnect.restype=c_int;
        this.TDeviceIsConnect.argtypes=[c_void_p]

        #TDEVICE_API TED_RESULT TDeviceSendTxtCmd(TDEVICE_HDL hdl, const TED_CHAR* cmd,  /*OUT*/ TED_CHAR* resp, TED_INT respMaxSize, TED_INT respDurMileSecond);
        this.TDeviceSendTxtCmd = Load_DLL['TDeviceSendTxtCmd']
        this.TDeviceSendTxtCmd.restype=c_int;
        this.TDeviceSendTxtCmd.argtypes=[c_void_p, c_char_p, c_char_p, c_int, c_int]

        #TDEVICE_API TED_RESULT TDeviceSendCtrlCmd(TDEVICE_HDL hdl, const TED_CHAR* cmd,  /*OUT*/ TED_CHAR* resp, TED_INT respMaxSize, TED_INT respDurMileSecond);
        this.TDeviceSendCtrlCmd = Load_DLL['TDeviceSendCtrlCmd']
        this.TDeviceSendCtrlCmd.restype=c_int;
        this.TDeviceSendCtrlCmd.argtypes=[c_void_p, c_char_p, c_char_p, c_int, c_int]

        #TDEVICE_API TED_RESULT TDeviceReadRegValue(TDEVICE_HDL hdl, TED_REGADDR regAddr, TED_INT byteOffset, TED_INT readCnt, /*OUT*/ TED_REGVALUE* regValue);
        this.TDeviceDD_DSIM_MipiReadReg = Load_DLL['TDeviceDD_DSIM_MipiReadReg']
        this.TDeviceDD_DSIM_MipiReadReg.restype=c_int;
        this.TDeviceDD_DSIM_MipiReadReg.argtypes=[c_void_p, c_char, c_int, c_int, c_char_p]

        #TDEVICE_API TED_RESULT TDeviceReadRegValue1Byte(TDEVICE_HDL hdl, TED_REGADDR regAddr, TED_INT byteOffset, /*OUT*/TED_REGVALUE* regValue);
        this.TDeviceDD_DSIM_MipiReadReg1Byte = Load_DLL['TDeviceDD_DSIM_MipiReadReg1Byte']
        this.TDeviceDD_DSIM_MipiReadReg1Byte.restype=c_int;
        this.TDeviceDD_DSIM_MipiReadReg1Byte.argtypes=[c_void_p, c_char, c_int, c_char_p]

        #TDEVICE_API TED_RESULT TDeviceWriteRegValue(TDEVICE_HDL hdl, TED_REGADDR regAddr, TED_INT byteOffset, TED_INT writeCnt,  TED_REGVALUE* regValue);
        this.TDeviceDD_DSIM_MipiWriteReg = Load_DLL['TDeviceDD_DSIM_MipiWriteReg']
        this.TDeviceDD_DSIM_MipiWriteReg.restype=c_int;
        this.TDeviceDD_DSIM_MipiWriteReg.argtypes=[c_void_p, c_char, c_int, c_int, c_char_p]

        #TDEVICE_API TED_RESULT TDeviceWriteRegValue1Byte(TDEVICE_HDL hdl, TED_REGADDR regAddr, TED_INT byteOffset, TED_REGVALUE regValue);
        this.TDeviceDD_DSIM_MipiWriteReg1Byte = Load_DLL['TDeviceDD_DSIM_MipiWriteReg1Byte']
        this.TDeviceDD_DSIM_MipiWriteReg1Byte.restype=c_int;
        this.TDeviceDD_DSIM_MipiWriteReg1Byte.argtypes=[c_void_p, c_char, c_int, c_char]

        #TDEVICE_API TED_BOOL TDeviceDD_DSIM_MipiWriteReg39(TDEVICE_HDL hdl, unsigned char regAddr, int writeCnt, unsigned char* regValueArr);
        this.TDeviceDD_DSIM_MipiWriteReg39 = Load_DLL['TDeviceDD_DSIM_MipiWriteReg39']
        this.TDeviceDD_DSIM_MipiWriteReg39.restype=c_int;
        this.TDeviceDD_DSIM_MipiWriteReg39.argtypes=[c_void_p, c_char, c_int, c_char_p]

        #TDEVICE_API TED_BOOL TDeviceDD_DSIM_MipiWriteReg15(TDEVICE_HDL hdl, unsigned char regAddr, unsigned char regValue);
        this.TDeviceDD_DSIM_MipiWriteReg15 = Load_DLL['TDeviceDD_DSIM_MipiWriteReg15']
        this.TDeviceDD_DSIM_MipiWriteReg15.restype=c_int;
        this.TDeviceDD_DSIM_MipiWriteReg15.argtypes=[c_void_p, c_char, c_char]

        #TDEVICE_API TED_BOOL TDeviceDD_DSIM_MipiWriteReg05(TDEVICE_HDL hdl, unsigned char regAddr);
        this.TDeviceDD_DSIM_MipiWriteReg05 = Load_DLL['TDeviceDD_DSIM_MipiWriteReg05']
        this.TDeviceDD_DSIM_MipiWriteReg05.restype=c_int;
        this.TDeviceDD_DSIM_MipiWriteReg05.argtypes=[c_void_p, c_char]

        #TDEVICE_API TED_BOOL TDeviceDD_DSIM_MipiWriteReg07(TDEVICE_HDL hdl, unsigned char value);
        this.TDeviceDD_DSIM_MipiWriteReg07 = Load_DLL['TDeviceDD_DSIM_MipiWriteReg07']
        this.TDeviceDD_DSIM_MipiWriteReg07.restype=c_int;
        this.TDeviceDD_DSIM_MipiWriteReg07.argtypes=[c_void_p, c_char]

        #TDEVICE_API TED_RESULT TDeviceCatchPowerInfo(TDEVICE_HDL hdl, /*OUT*/struct TED_POWER_INFO* p_pwrinfo, TED_INT timeOut /*milisec */)
        this.TDeviceCatchPowerInfo = Load_DLL['TDeviceCatchPowerInfo']
        this.TDeviceCatchPowerInfo.restype=c_int;
        this.TDeviceCatchPowerInfo.argtypes=[c_void_p, c_char_p, c_int]

        #TDEVICE_API TED_BOOL TDeviceDebugMessage(TDEVICE_HDL hdl, const char* msg);
        this.TDeviceDebugMessage = Load_DLL['TDeviceDebugMessage']
        this.TDeviceDebugMessage.restype=c_int;
        this.TDeviceDebugMessage.argtypes=[c_void_p, c_char_p]

        
        #TDEVICE_API TED_BOOL TDevicePatternConnect(void* hdl);
        this.TDevicePatternConnect = Load_DLL['TDevicePatternConnect']
        this.TDevicePatternConnect.restype=c_int;
        this.TDevicePatternConnect.argtypes=[c_void_p]

        #TDEVICE_API TED_BOOL TDevicePatternIsConnect(void* hdl);
        this.TDevicePatternIsConnect = Load_DLL['TDevicePatternIsConnect']
        this.TDevicePatternIsConnect.restype=c_int;
        this.TDevicePatternIsConnect.argtypes=[c_void_p]

        #TDEVICE_API TED_BOOL TDevicePatternDisconnect(void* hdl);
        this.TDevicePatternDisconnect = Load_DLL['TDevicePatternDisconnect']
        this.TDevicePatternDisconnect.restype=c_int;
        this.TDevicePatternDisconnect.argtypes=[c_void_p]

        #TDEVICE_API TED_BOOL TDevicePatternPaint(void* hdl, unsigned char r, unsigned char g, unsigned char b, unsigned char a);
        this.TDevicePatternPaint = Load_DLL['TDevicePatternPaint']
        this.TDevicePatternPaint.restype=c_int;
        this.TDevicePatternPaint.argtypes=[c_void_p, c_int, c_int, c_int, c_int]
                
        #TDEVICE_API TED_BOOL TDevicePatternUpdateScreen(void* hdl);
        this.TDevicePatternUpdateScreen = Load_DLL['TDevicePatternUpdateScreen']
        this.TDevicePatternUpdateScreen.restype=c_int;
        this.TDevicePatternUpdateScreen.argtypes=[c_void_p]
        
        #TDEVICE_API TED_BOOL TDevicePatternSetCommand(void* hdl, const char* cmd);
        this.TDevicePatternSetCommand = Load_DLL['TDevicePatternSetCommand']
        this.TDevicePatternSetCommand.restype=c_int;
        this.TDevicePatternSetCommand.argtypes=[c_void_p, c_char_p]

        #TDEVICE_API TED_BOOL TDevicePatternDrawImage(void* hdl, const char* imgFileName);
        this.TDevicePatternDrawImage = Load_DLL['TDevicePatternDrawImage']
        this.TDevicePatternDrawImage.restype=c_int;
        this.TDevicePatternDrawImage.argtypes=[c_void_p, c_char_p]

        #TDEVICE_API TED_BOOL TDevicePatternScreenVerify(void* hdl, unsigned char r, unsigned char g, unsigned char b);
        this.TDevicePatternScreenVerify = Load_DLL['TDevicePatternScreenVerify']
        this.TDevicePatternScreenVerify.restype=c_int;
        this.TDevicePatternScreenVerify.argtypes=[c_void_p, c_int, c_int, c_int]


        #TDEVICE_API TED_BOOL TDeviceDebugFuncEnter(TDEVICE_HDL hdl, const char* funcName);
        this.TDeviceDebugFuncEnter = Load_DLL['TDeviceDebugFuncEnter']
        this.TDeviceDebugFuncEnter.restype=c_int;
        this.TDeviceDebugFuncEnter.argtypes=[c_void_p, c_char_p]

        #TDEVICE_API TED_BOOL TDeviceDebugFuncLeave(TDEVICE_HDL hdl, const char* funcName);
        this.TDeviceDebugFuncLeave = Load_DLL['TDeviceDebugFuncLeave']
        this.TDeviceDebugFuncLeave.restype=c_int;
        this.TDeviceDebugFuncLeave.argtypes=[c_void_p, c_char_p]


        #TDEVICE_API TED_BOOL TDeviceDD_DSIM_manual_ctrl(void* hdl, int dutIdx, int value);
        this.TDeviceDD_DSIM_manual_ctrl = Load_DLL['TDeviceDD_DSIM_manual_ctrl']
        this.TDeviceDD_DSIM_manual_ctrl.restype=c_int;
        this.TDeviceDD_DSIM_manual_ctrl.argtypes=[c_void_p, c_int, c_int]

        #TDEVICE_API TED_BOOL TDeviceDD_DSIM_power_ctrl(void* hdl, int dutIdx, int value);
        this.TDeviceDD_DSIM_power_ctrl = Load_DLL['TDeviceDD_DSIM_power_ctrl']
        this.TDeviceDD_DSIM_power_ctrl.restype=c_int;
        this.TDeviceDD_DSIM_power_ctrl.argtypes=[c_void_p, c_int, c_int]

        #TDEVICE_API TED_BOOL TDeviceDD_DSIM_source_cal(void* hdl, int dutIdx, int value);
        this.TDeviceDD_DSIM_source_cal = Load_DLL['TDeviceDD_DSIM_source_cal']
        this.TDeviceDD_DSIM_source_cal.restype=c_int;
        this.TDeviceDD_DSIM_source_cal.argtypes=[c_void_p, c_int, c_int]

        #TDEVICE_API TED_BOOL TDeviceDD_DSIM_sleepin(void* hdl, int dutIdx, int value);
        this.TDeviceDD_DSIM_sleepin = Load_DLL['TDeviceDD_DSIM_sleepin']
        this.TDeviceDD_DSIM_sleepin.restype=c_int;
        this.TDeviceDD_DSIM_sleepin.argtypes=[c_void_p, c_int, c_int]

        #TDEVICE_API TED_BOOL TDeviceDD_DSIM_sleepout(void* hdl, int dutIdx, int value);
        this.TDeviceDD_DSIM_sleepout = Load_DLL['TDeviceDD_DSIM_sleepout']
        this.TDeviceDD_DSIM_sleepout.restype=c_int;
        this.TDeviceDD_DSIM_sleepout.argtypes=[c_void_p, c_int, c_int]

        #TDEVICE_API TED_BOOL TDeviceDD_DSIM_deep_standby(void* hdl, int dutIdx, int value);
        this.TDeviceDD_DSIM_deep_standby = Load_DLL['TDeviceDD_DSIM_deep_standby']
        this.TDeviceDD_DSIM_deep_standby.restype=c_int;
        this.TDeviceDD_DSIM_deep_standby.argtypes=[c_void_p, c_int, c_int]

        #TDEVICE_API TED_BOOL TDeviceDD_DSIM_displayon(void* hdl, int dutIdx, int value);
        this.TDeviceDD_DSIM_displayon = Load_DLL['TDeviceDD_DSIM_displayon']
        this.TDeviceDD_DSIM_displayon.restype=c_int;
        this.TDeviceDD_DSIM_displayon.argtypes=[c_void_p, c_int, c_int]

        #TDEVICE_API TED_BOOL TDeviceDD_DSIM_reset_ctrl(void* hdl, int dutIdx, int value);
        this.TDeviceDD_DSIM_reset_ctrl = Load_DLL['TDeviceDD_DSIM_reset_ctrl']
        this.TDeviceDD_DSIM_reset_ctrl.restype=c_int;
        this.TDeviceDD_DSIM_reset_ctrl.argtypes=[c_void_p, c_int, c_int]

        #TDEVICE_API TED_BOOL TDeviceDD_FB_blank(void* hdl, int dutIdx, int value);
        this.TDeviceDD_FB_blank = Load_DLL['TDeviceDD_FB_blank']
        this.TDeviceDD_FB_blank.restype=c_int;
        this.TDeviceDD_FB_blank.argtypes=[c_void_p, c_int, c_int]

        #//VLIN1_ADC
        #TDEVICE_API TED_BOOL TDeviceVlin1AdcSetSamples(TDEVICE_HDL hdl, int dutIdx, int value);  
        this.TDeviceVlin1AdcSetSamples = Load_DLL['TDeviceVlin1AdcSetSamples']
        this.TDeviceVlin1AdcSetSamples.restype=c_int;
        this.TDeviceVlin1AdcSetSamples.argtypes=[c_void_p, c_int, c_int]

        #TDEVICE_API TED_BOOL TDeviceVlin1AdcSetInterval(TDEVICE_HDL hdl,  int dutIdx,  int value); 
        this.TDeviceVlin1AdcSetInterval = Load_DLL['TDeviceVlin1AdcSetInterval']
        this.TDeviceVlin1AdcSetInterval.restype=c_int;
        this.TDeviceVlin1AdcSetInterval.argtypes=[c_void_p, c_int, c_int]

        #TDEVICE_API TED_BOOL TDeviceVlin1AdcSetChannelOn(TDEVICE_HDL hdl, int dutIdx, int chIdx);
        this.TDeviceVlin1AdcSetChannelOn = Load_DLL['TDeviceVlin1AdcSetChannelOn']
        this.TDeviceVlin1AdcSetChannelOn.restype=c_int;
        this.TDeviceVlin1AdcSetChannelOn.argtypes=[c_void_p, c_int, c_int]

        #TDEVICE_API TED_BOOL TDeviceVlin1AdcSetChannelOff(TDEVICE_HDL hdl,  int dutIdx, int chIdx); 
        this.TDeviceVlin1AdcSetChannelOff = Load_DLL['TDeviceVlin1AdcSetChannelOff']
        this.TDeviceVlin1AdcSetChannelOff.restype=c_int;
        this.TDeviceVlin1AdcSetChannelOff.argtypes=[c_void_p, c_int, c_int]

        #TDEVICE_API TED_BOOL TDeviceVlin1AdcSetMode(TDEVICE_HDL hdl,  int dutIdx, int value);  
        this.TDeviceVlin1AdcSetMode = Load_DLL['TDeviceVlin1AdcSetMode']
        this.TDeviceVlin1AdcSetMode.restype=c_int;
        this.TDeviceVlin1AdcSetMode.argtypes=[c_void_p, c_int, c_int]

        #TDEVICE_API int TDeviceVlin1AdcGetVoltage(TDEVICE_HDL hdl,  int dutIdx, int chIdx);  
        this.TDeviceVlin1AdcGetVoltage = Load_DLL['TDeviceVlin1AdcGetVoltage']
        this.TDeviceVlin1AdcGetVoltage.restype=c_int;
        this.TDeviceVlin1AdcGetVoltage.argtypes=[c_void_p, c_int, c_int]

        #TDEVICE_API int TDeviceVlin1AdcGetVoltageVLIN1(TDEVICE_HDL hdl, int dutIdx, );  
        this.TDeviceVlin1AdcGetVoltageVLIN1 = Load_DLL['TDeviceVlin1AdcGetVoltageVLIN1']
        this.TDeviceVlin1AdcGetVoltageVLIN1.restype=c_int;
        this.TDeviceVlin1AdcGetVoltageVLIN1.argtypes=[c_void_p, c_int]

        #TDEVICE_API int TDeviceVlin1AdcGetVoltageVBAT(TDEVICE_HDL hdl,  int dutIdx, );  
        this.TDeviceVlin1AdcGetVoltageVBAT = Load_DLL['TDeviceVlin1AdcGetVoltageVBAT']
        this.TDeviceVlin1AdcGetVoltageVBAT.restype=c_int;
        this.TDeviceVlin1AdcGetVoltageVBAT.argtypes=[c_void_p, c_int]

        #TDEVICE_API int TDeviceVlin1AdcGetVoltageELVDD(TDEVICE_HDL hdl,  int dutIdx, );  
        this.TDeviceVlin1AdcGetVoltageELVDD = Load_DLL['TDeviceVlin1AdcGetVoltageELVDD']
        this.TDeviceVlin1AdcGetVoltageELVDD.restype=c_int;
        this.TDeviceVlin1AdcGetVoltageELVDD.argtypes=[c_void_p, c_int]

        #TDEVICE_API int TDeviceVlin1AdcGetCurrent(TDEVICE_HDL hdl,  int dutIdx, int chIdx);  
        this.TDeviceVlin1AdcGetCurrent = Load_DLL['TDeviceVlin1AdcGetCurrent']
        this.TDeviceVlin1AdcGetCurrent.restype=c_int;
        this.TDeviceVlin1AdcGetCurrent.argtypes=[c_void_p, c_int, c_int]

        #TDEVICE_API int TDeviceVlin1AdcGetCurrentVLIN1(TDEVICE_HDL hdl,  int dutIdx, ); 
        this.TDeviceVlin1AdcGetCurrentVLIN1 = Load_DLL['TDeviceVlin1AdcGetCurrentVLIN1']
        this.TDeviceVlin1AdcGetCurrentVLIN1.restype=c_int;
        this.TDeviceVlin1AdcGetCurrentVLIN1.argtypes=[c_void_p, c_int]

        #TDEVICE_API int TDeviceVlin1AdcGetCurrentVBAT(TDEVICE_HDL hdl,  int dutIdx, ); 
        this.TDeviceVlin1AdcGetCurrentVBAT = Load_DLL['TDeviceVlin1AdcGetCurrentVBAT']
        this.TDeviceVlin1AdcGetCurrentVBAT.restype=c_int;
        this.TDeviceVlin1AdcGetCurrentVBAT.argtypes=[c_void_p, c_int]

        #TDEVICE_API int TDeviceVlin1AdcGetCurrentELVDD(TDEVICE_HDL hdl,  int dutIdx, ); 
        this.TDeviceVlin1AdcGetCurrentELVDD = Load_DLL['TDeviceVlin1AdcGetCurrentELVDD']
        this.TDeviceVlin1AdcGetCurrentELVDD.restype=c_int;
        this.TDeviceVlin1AdcGetCurrentELVDD.argtypes=[c_void_p, c_int]

        #//VCI_ADC
        #TDEVICE_API TED_BOOL TDeviceVciAdcSetSamples(TDEVICE_HDL hdl,  int dutIdx, int value); 
        this.TDeviceVciAdcSetSamples = Load_DLL['TDeviceVciAdcSetSamples']
        this.TDeviceVciAdcSetSamples.restype=c_int;
        this.TDeviceVciAdcSetSamples.argtypes=[c_void_p, c_int, c_int]

        #TDEVICE_API TED_BOOL TDeviceVciAdcSetInterval(TDEVICE_HDL hdl,  int dutIdx, int value);
        this.TDeviceVciAdcSetInterval = Load_DLL['TDeviceVciAdcSetInterval']
        this.TDeviceVciAdcSetInterval.restype=c_int;
        this.TDeviceVciAdcSetInterval.argtypes=[c_void_p, c_int, c_int]

        #TDEVICE_API TED_BOOL TDeviceVciAdcSetChannelOn(TDEVICE_HDL hdl,  int dutIdx, int chIdx);
        this.TDeviceVciAdcSetChannelOn = Load_DLL['TDeviceVciAdcSetChannelOn']
        this.TDeviceVciAdcSetChannelOn.restype=c_int;
        this.TDeviceVciAdcSetChannelOn.argtypes=[c_void_p, c_int, c_int]

        #TDEVICE_API TED_BOOL TDeviceVciAdcSetChannelOff(TDEVICE_HDL hdl,  int dutIdx, int chIdx);
        this.TDeviceVciAdcSetChannelOff = Load_DLL['TDeviceVciAdcSetChannelOff']
        this.TDeviceVciAdcSetChannelOff.restype=c_int;
        this.TDeviceVciAdcSetChannelOff.argtypes=[c_void_p, c_int, c_int]

        #TDEVICE_API TED_BOOL TDeviceVciAdcSetMode(TDEVICE_HDL hdl,  int dutIdx, int value);
        this.TDeviceVciAdcSetMode = Load_DLL['TDeviceVciAdcSetMode']
        this.TDeviceVciAdcSetMode.restype=c_int;
        this.TDeviceVciAdcSetMode.argtypes=[c_void_p, c_int, c_int]

        #TDEVICE_API int TDeviceVciAdcGetVoltage(TDEVICE_HDL hdl,  int dutIdx, int chIdx);
        this.TDeviceVciAdcGetVoltage = Load_DLL['TDeviceVciAdcGetVoltage']
        this.TDeviceVciAdcGetVoltage.restype=c_int;
        this.TDeviceVciAdcGetVoltage.argtypes=[c_void_p, c_int, c_int]

        #TDEVICE_API int TDeviceVciAdcGetVoltageVCI(TDEVICE_HDL hdl,  int dutIdx, );
        this.TDeviceVciAdcGetVoltageVCI = Load_DLL['TDeviceVciAdcGetVoltageVCI']
        this.TDeviceVciAdcGetVoltageVCI.restype=c_int;
        this.TDeviceVciAdcGetVoltageVCI.argtypes=[c_void_p, c_int]

        #TDEVICE_API int TDeviceVciAdcGetVoltageVDDR(TDEVICE_HDL hdl,  int dutIdx, );
        this.TDeviceVciAdcGetVoltageVDDR = Load_DLL['TDeviceVciAdcGetVoltageVDDR']
        this.TDeviceVciAdcGetVoltageVDDR.restype=c_int;
        this.TDeviceVciAdcGetVoltageVDDR.argtypes=[c_void_p, c_int]

        #TDEVICE_API int TDeviceVciAdcGetVoltageVDDI(TDEVICE_HDL hdl,  int dutIdx, );
        this.TDeviceVciAdcGetVoltageVDDI = Load_DLL['TDeviceVciAdcGetVoltageVDDI']
        this.TDeviceVciAdcGetVoltageVDDI.restype=c_int;
        this.TDeviceVciAdcGetVoltageVDDI.argtypes=[c_void_p, c_int]

        #TDEVICE_API int TDeviceVciAdcGetCurrent(TDEVICE_HDL hdl,  int dutIdx, int chIdx);
        this.TDeviceVciAdcGetCurrent = Load_DLL['TDeviceVciAdcGetCurrent']
        this.TDeviceVciAdcGetCurrent.restype=c_int;
        this.TDeviceVciAdcGetCurrent.argtypes=[c_void_p, c_int, c_int]

        #TDEVICE_API int TDeviceVciAdcGetCurrentVCI(TDEVICE_HDL hdl,  int dutIdx, );
        this.TDeviceVciAdcGetCurrentVCI = Load_DLL['TDeviceVciAdcGetCurrentVCI']
        this.TDeviceVciAdcGetCurrentVCI.restype=c_int;
        this.TDeviceVciAdcGetCurrentVCI.argtypes=[c_void_p, c_int]

        #TDEVICE_API int TDeviceVciAdcGetCurrentVDDR(TDEVICE_HDL hdl,  int dutIdx, );
        this.TDeviceVciAdcGetCurrentVDDR = Load_DLL['TDeviceVciAdcGetCurrentVDDR']
        this.TDeviceVciAdcGetCurrentVDDR.restype=c_int;
        this.TDeviceVciAdcGetCurrentVDDR.argtypes=[c_void_p, c_int]

        #TDEVICE_API int TDeviceVciAdcGetCurrentVDDI(TDEVICE_HDL hdl,  int dutIdx, );
        this.TDeviceVciAdcGetCurrentVDDI = Load_DLL['TDeviceVciAdcGetCurrentVDDI']
        this.TDeviceVciAdcGetCurrentVDDI.restype=c_int;
        this.TDeviceVciAdcGetCurrentVDDI.argtypes=[c_void_p, c_int]

        #//SDOUT ADC
        #define TED_SDOUTADC_MAX_CH_COUNT 16
        #TDEVICE_API int TDeviceSoutAdcSetDevConfig(TDEVICE_HDL hdl,  int dutIdx, int value);
        this.TDeviceSoutAdcSetDevConfig = Load_DLL['TDeviceSoutAdcSetDevConfig']
        this.TDeviceSoutAdcSetDevConfig.restype=c_int;
        this.TDeviceSoutAdcSetDevConfig.argtypes=[c_void_p, c_int, c_int]

        #TDEVICE_API int TDeviceSoutAdcSetInConfig(TDEVICE_HDL hdl,  int dutIdx, int chIdx, int value); 
        this.TDeviceSoutAdcSetInConfig = Load_DLL['TDeviceSoutAdcSetInConfig']
        this.TDeviceSoutAdcSetInConfig.restype=c_int;
        this.TDeviceSoutAdcSetInConfig.argtypes=[c_void_p, c_int, c_int, c_int]

        #TDEVICE_API int TDeviceSoutAdcSetRBSel(TDEVICE_HDL hdl,  int dutIdx, int value);    
        this.TDeviceSoutAdcSetRBSel = Load_DLL['TDeviceSoutAdcSetRBSel']
        this.TDeviceSoutAdcSetRBSel.restype=c_int;
        this.TDeviceSoutAdcSetRBSel.argtypes=[c_void_p, c_int, c_int]

        #TDEVICE_API int TDeviceSoutAdcGetChannelCount(TDEVICE_HDL hdl,  int dutIdx, );
        this.TDeviceSoutAdcGetChannelCount = Load_DLL['TDeviceSoutAdcGetChannelCount']
        this.TDeviceSoutAdcGetChannelCount.restype=c_int;
        this.TDeviceSoutAdcGetChannelCount.argtypes=[c_void_p, c_int]

        #TDEVICE_API int TDeviceSoutAdcGetVoltage(TDEVICE_HDL hdl,  int dutIdx, int chIdx);  
        this.TDeviceSoutAdcGetVoltage = Load_DLL['TDeviceSoutAdcGetVoltage']
        this.TDeviceSoutAdcGetVoltage.restype=c_int;
        this.TDeviceSoutAdcGetVoltage.argtypes=[c_void_p, c_int, c_int]

        #TDEVICE_API TED_BOOL TDeviceSoutAdcGetAllVoltage(TDEVICE_HDL hdl,  int dutIdx, int* voltageArray);
        this.TDeviceSoutAdcGetAllVoltage = Load_DLL['TDeviceSoutAdcGetAllVoltage']
        this.TDeviceSoutAdcGetAllVoltage.restype=c_int;
        this.TDeviceSoutAdcGetAllVoltage.argtypes=[c_void_p, c_int, c_char_p]


        #TDEVICE_API int TDeviceLdoAdcSetInConfig(TDEVICE_HDL hdl,  int dutIdx, int chIdx, int value); 
        this.TDeviceLdoAdcSetInConfig = Load_DLL['TDeviceLdoAdcSetInConfig']
        this.TDeviceLdoAdcSetInConfig.restype=c_int;
        this.TDeviceLdoAdcSetInConfig.argtypes=[c_void_p, c_int, c_int, c_int]

        #TDEVICE_API int TDeviceLdoAdcGetChannelCount(TDEVICE_HDL hdl,  int dutIdx, );
        this.TDeviceLdoAdcGetChannelCount = Load_DLL['TDeviceLdoAdcGetChannelCount']
        this.TDeviceLdoAdcGetChannelCount.restype=c_int;
        this.TDeviceLdoAdcGetChannelCount.argtypes=[c_void_p, c_int]

        #TDEVICE_API int TDeviceLdoAdcGetVoltage(TDEVICE_HDL hdl,  int dutIdx, int chIdx);  
        this.TDeviceLdoAdcGetVoltage = Load_DLL['TDeviceLdoAdcGetVoltage']
        this.TDeviceLdoAdcGetVoltage.restype=c_int;
        this.TDeviceLdoAdcGetVoltage.argtypes=[c_void_p, c_int, c_int]

        #TDEVICE_API TED_BOOL TDeviceLdoAdcGetAllVoltage(TDEVICE_HDL hdl,  int dutIdx, int* voltageArray);
        this.TDeviceLdoAdcGetAllVoltage = Load_DLL['TDeviceLdoAdcGetAllVoltage']
        this.TDeviceLdoAdcGetAllVoltage.restype=c_int;
        this.TDeviceLdoAdcGetAllVoltage.argtypes=[c_void_p, c_int, c_char_p]


        #TDEVICE_API int TDeviceRegAdcSetInConfig(TDEVICE_HDL hdl,  int dutIdx, int chIdx, int value); 
        this.TDeviceRegAdcSetInConfig = Load_DLL['TDeviceRegAdcSetInConfig']
        this.TDeviceRegAdcSetInConfig.restype=c_int;
        this.TDeviceRegAdcSetInConfig.argtypes=[c_void_p, c_int, c_int, c_int]

        #TDEVICE_API int TDeviceRegAdcGetChannelCount(TDEVICE_HDL hdl, int dutIdx);
        this.TDeviceRegAdcGetChannelCount = Load_DLL['TDeviceRegAdcGetChannelCount']
        this.TDeviceRegAdcGetChannelCount.restype=c_int;
        this.TDeviceRegAdcGetChannelCount.argtypes=[c_void_p, c_int]

        #TDEVICE_API int TDeviceRegAdcGetVoltage(TDEVICE_HDL hdl,  int dutIdx, int chIdx);  
        this.TDeviceRegAdcGetVoltage = Load_DLL['TDeviceRegAdcGetVoltage']
        this.TDeviceRegAdcGetVoltage.restype=c_int;
        this.TDeviceRegAdcGetVoltage.argtypes=[c_void_p, c_int, c_int]

        #TDEVICE_API TED_BOOL TDeviceRegAdcGetAllVoltage(TDEVICE_HDL hdl,  int dutIdx, int* voltageArray);
        this.TDeviceRegAdcGetAllVoltage = Load_DLL['TDeviceRegAdcGetAllVoltage']
        this.TDeviceRegAdcGetAllVoltage.restype=c_int;
        this.TDeviceRegAdcGetAllVoltage.argtypes=[c_void_p, c_int, c_char_p]

        #TDEVICE_API TED_BOOL TDeviceAgingNotifyPyStart(TDEVICE_HDL hdl); 
        this.TDeviceAgingNotifyPyStart = Load_DLL['TDeviceAgingNotifyPyStart']
        this.TDeviceAgingNotifyPyStart.restype=c_int;
        this.TDeviceAgingNotifyPyStart.argtypes=[c_void_p, c_char_p]

        #TDEVICE_API TED_BOOL TDeviceAgingNotifyPyStop(TDEVICE_HDL hdl); 
        this.TDeviceAgingNotifyPyStop = Load_DLL['TDeviceAgingNotifyPyStop']
        this.TDeviceAgingNotifyPyStop.restype=c_int;
        this.TDeviceAgingNotifyPyStop.argtypes=[c_void_p, c_char_p]

        #TDEVICE_API TED_BOOL TDeviceAgingSetCurJobInfo(void* hdl, int dutIdx, int jobID, int status, int scIdx, int scCount, const char* desc);
        this.TDeviceAgingSetCurJobInfo = Load_DLL['TDeviceAgingSetCurJobInfo']
        this.TDeviceAgingSetCurJobInfo.restype=c_int;
        this.TDeviceAgingSetCurJobInfo.argtypes=[c_void_p, c_int, c_int, c_int, c_int, c_int, c_char_p]

        #TDEVICE_API TED_BOOL TDeviceAgingSetCurScInfo(void* hdl, int dutIdx, int scID, int status, int tcIdx, int tcCount, const char* desc);
        this.TDeviceAgingSetCurScInfo = Load_DLL['TDeviceAgingSetCurScInfo']
        this.TDeviceAgingSetCurScInfo.restype=c_int;
        this.TDeviceAgingSetCurScInfo.argtypes=[c_void_p, c_int, c_int, c_int, c_int, c_int, c_char_p]

        #TDEVICE_API TED_BOOL TDeviceAgingSetCurTcInfo(void* hdl, int dutIdx, int tcID, int status, int tcStepIdx, int tcStepCount, const char* desc);
        this.TDeviceAgingSetCurTcInfo = Load_DLL['TDeviceAgingSetCurTcInfo']
        this.TDeviceAgingSetCurTcInfo.restype=c_int;
        this.TDeviceAgingSetCurTcInfo.argtypes=[c_void_p, c_int, c_int, c_int, c_int, c_int, c_char_p]

        #TDEVICE_API TED_BOOL TDeviceAgingSetCurTcStepInfo(void* hdl, int dutIdx, int tcStepID, int status, const char* desc);
        this.TDeviceAgingSetCurTcStepInfo = Load_DLL['TDeviceAgingSetCurTcStepInfo']
        this.TDeviceAgingSetCurTcStepInfo.restype=c_int;
        this.TDeviceAgingSetCurTcStepInfo.argtypes=[c_void_p, c_int, c_int, c_int, c_char_p]

        #TDEVICE_API TED_BOOL TDeviceAgingMeasureADC(void* hdl, int dutIdx, /*OUT*/ void* res);
        this.TDeviceAgingMeasureADC = Load_DLL['TDeviceAgingMeasureADC']
        this.TDeviceAgingMeasureADC.restype=c_int;
        this.TDeviceAgingMeasureADC.argtypes=[c_void_p, c_int, c_char_p]

        #TDEVICE_API int TDeviceAgingMeasureADCResultStructureByteSize(void* hdl);
        this.TDeviceAgingMeasureADCResultStructureByteSize = Load_DLL['TDeviceAgingMeasureADCResultStructureByteSize']
        this.TDeviceAgingMeasureADCResultStructureByteSize.restype=c_int;
        this.TDeviceAgingMeasureADCResultStructureByteSize.argtypes=[c_void_p]


        # enum TFileTransferType {
        #    TFileTransferTypeT5 = 0,
        #    TFileTransferTypeMaxCount
        #};

        # enum TFileTransferError {
        #    TFileTransferErrorSuccess = 0,
        #    TFileTransferErrorSendPacket,
        #    TFileTransferErrorNoResp,
        #    TFileTransferErrorFileOpen,
        #    TFileTransferErrorStorageSize,
        #    TFileTransferErrorCRC
        #};

        #TDEVICE_API TFILETRANSFER_HDL TFileTransferCreate(enum TFileTransferType type, TDEVICE_HDL deviceHandle);
        #TDEVICE_API TED_BOOL TFileTransferDestroy(TFILETRANSFER_HDL fileTransferHandle);
        #TDEVICE_API TED_BOOL TFileTransferStart(TFILETRANSFER_HDL fileTransferHandle, const char* fileName);
        #TDEVICE_API TED_BOOL TFileTransferStop(TFILETRANSFER_HDL fileTransferHandle);
        #TDEVICE_API int TFileTransferGetFileByteSize(TFILETRANSFER_HDL fileTransferHandle);
        #TDEVICE_API int TFileTransferGetTransferByteSize(TFILETRANSFER_HDL fileTransferHandle);
        #TDEVICE_API TED_BOOL TFileTransferIsStart(TFILETRANSFER_HDL fileTransferHandle);
        #TDEVICE_API TED_BOOL TFileTransferIsDone(TFILETRANSFER_HDL fileTransferHandle);
        #TDEVICE_API TED_BOOL TFileTransferIsError(TFILETRANSFER_HDL fileTransferHandle);
        #TDEVICE_API enum TFileTransferError TFileTransferGetLastError(TFILETRANSFER_HDL fileTransferHandle);

        #TDEVICE_API TFILETRANSFER_HDL TFileTransferCreate(enum TFileTransferType type, TDEVICE_HDL deviceHandle);
        this.TFileTransferCreate = Load_DLL['TFileTransferCreate']
        this.TFileTransferCreate.restype=c_void_p;
        this.TFileTransferCreate.argtypes=[c_int, c_void_p]

        #TDEVICE_API TED_BOOL TFileTransferDestroy(TFILETRANSFER_HDL fileTransferHandle);
        this.TFileTransferDestroy=Load_DLL['TFileTransferDestroy']
        this.TFileTransferDestroy.restype=c_int;
        this.TFileTransferDestroy.argtypes=[c_void_p]

        #TDEVICE_API TED_BOOL TFileTransferStart(TFILETRANSFER_HDL fileTransferHandle, const char* fileName);
        this.TFileTransferStart=Load_DLL['TFileTransferStart']
        this.TFileTransferStart.restype=c_int;
        this.TFileTransferStart.argtypes=[c_void_p, c_void_p]

        #TDEVICE_API TED_BOOL TFileTransferStop(TFILETRANSFER_HDL fileTransferHandle);
        this.TFileTransferStop=Load_DLL['TFileTransferStop']
        this.TFileTransferStop.restype=c_int;
        this.TFileTransferStop.argtypes=[c_void_p]

        #TDEVICE_API int TFileTransferGetFileByteSize(TFILETRANSFER_HDL fileTransferHandle);
        this.TFileTransferGetFileByteSize=Load_DLL['TFileTransferGetFileByteSize']
        this.TFileTransferGetFileByteSize.restype=c_int;
        this.TFileTransferGetFileByteSize.argtypes=[c_void_p]

        #TDEVICE_API int TFileTransferGetTransferByteSize(TFILETRANSFER_HDL fileTransferHandle);
        this.TFileTransferGetTransferByteSize=Load_DLL['TFileTransferGetTransferByteSize']
        this.TFileTransferGetTransferByteSize.restype=c_int;
        this.TFileTransferGetTransferByteSize.argtypes=[c_void_p]

        #TDEVICE_API TED_BOOL TFileTransferIsStart(TFILETRANSFER_HDL fileTransferHandle);
        this.TFileTransferIsStart=Load_DLL['TFileTransferIsStart']
        this.TFileTransferIsStart.restype=c_int;
        this.TFileTransferIsStart.argtypes=[c_void_p]

        #TDEVICE_API TED_BOOL TFileTransferIsDone(TFILETRANSFER_HDL fileTransferHandle);
        this.TFileTransferIsDone=Load_DLL['TFileTransferIsDone']
        this.TFileTransferIsDone.restype=c_int;
        this.TFileTransferIsDone.argtypes=[c_void_p]

        #TDEVICE_API TED_BOOL TFileTransferIsError(TFILETRANSFER_HDL fileTransferHandle);
        this.TFileTransferIsError=Load_DLL['TFileTransferIsError']
        this.TFileTransferIsError.restype=c_int;
        this.TFileTransferIsError.argtypes=[c_void_p]

        #TDEVICE_API enum TFileTransferError TFileTransferGetLastError(TFILETRANSFER_HDL fileTransferHandle);
        this.TFileTransferGetLastError=Load_DLL['TFileTransferGetLastError']
        this.TFileTransferGetLastError.restype=c_int;
        this.TFileTransferGetLastError.argtypes=[c_void_p]






from Anapass.TModule import *
from Anapass import FlashMemory


#T5 Device에 연결한다. 
device = TDevice(TDevice.Type.T5)
isOk = device.Connect()
if isOk != True :
    raise ConnectionRefusedError("Fail to connect to TDevice(%s)"%device.GetName())

#Flash Memory Instacne 생성
#  TChip.Type.ANA6705 :  Anapass/ANA6705/ANA6705_FlashMipiToSp.py 에  코드 구현됨 
#  TChip.Type.ANA6706 :  Anapass/ANA6706/ANA6706_FlashMipiToSp.py 에  코드 구현됨 
chipType = TChip.Type.ANA6705
#chipType = TChip.Type.ANA6706
flash = FlashMemory.CreateInstance(device, chipType, FlashMemory.Type.MipiToSpi)

eraseSectorAddr = 0x10 #  sector주소 0x10을 Erase한다 

eraseMemAddr = flash.ConvertSectorAddrToMemAddr(eraseSectorAddr)
sectorByteSize = flash.GetFlashSectorByteSize()

print("Erase SectorAddr 0x%08X" % eraseSectorAddr)
print("Erase SectorMem  0x%08X~0x%08X " % (eraseMemAddr, eraseMemAddr+sectorByteSize-1))

isOk = flash.EraseSector(eraseSectorAddr)
assert isOk 

readData=[0 for _ in range(sectorByteSize)]  #1 sector 만큼 읽는다.
isOk = flash.Read(eraseMemAddr, sectorByteSize, readData)
assert isOk 

#Sector Erase했으므로 읽은 모든 데이타가 0xFF 여야 한다.
verify=True
for data in readData :
    if data != 0xFF :
        verify=False
        break

if verify :
    print("Success : Erase Sector")
else :
    print("FAIL: Erase Sector")

#T5 Device 접속 종료
device.Disonnect()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="anapass-python", # Replace with your own username
    version="1.0.0.14",
    author="hthwang",
    author_email="hthwang@anapass.com",
    description="Anapass Python Package",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="http://www.anapass.com",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    data_files=["AnapassTComm.dll", "AnapassTComm32.dll", "AnapassTModule.dll", "AnapassTModule32.dll", "AnapassTParser.dll", "AnapassTParser32.dll" ]
)





from Anapass.TModule import *
from Anapass import FlashMemory


#T5 Device에 연결한다. 
device = TDevice(TDevice.Type.T5)
isOk = device.Connect()
if isOk != True :
    raise ConnectionRefusedError("Fail to connect to TDevice(%s)"%device.GetName())

#Flash Memory Instacne 생성
#  TChip.Type.ANA6705 :  Anapass/ANA6705/ANA6705_FlashMipiToSp.py 에  코드 구현됨 
#  TChip.Type.ANA6706 :  Anapass/ANA6706/ANA6706_FlashMipiToSp.py 에  코드 구현됨 
chipType = TChip.Type.ANA6705
#chipType = TChip.Type.ANA6706
flash = FlashMemory.CreateInstance(device, chipType, FlashMemory.Type.MipiToSpi)

status1 =  flash.ReadStatus1()
status2  = flash.ReadStatus2()
if status2&0x02 : 
    isLock = False
else :
    isLock = True

print("status1 =  0x%02X" % status1)
print("status2 =  0x%02X" % status2)
print("isLock =  " , isLock)

#T5 Device 접속 종료
device.Disonnect()




from Anapass.TModule import *

def ReadChipID(device) :
    #
    # Python프로그램을 Device를  연결하는 코드이다.
    # T4/T5 에 연결해서 사용하기 위해서는, 먼저  TEDTools의 TMonitor가 활성화되어야 한다. 
    # http://aits.anapass.com:8090/display/SI/TED+Tools  에서 UserManual Chap2. 'T4/T5구동기 연결하기' 참고 
    #
    #    
    print("Connect To " + device.GetName())
    isConnect = device.Connect()
    if isConnect != True :
        print("Connect Fail")
        quit()

    # ChipID를 읽는다.
    print("Read ChipID...")
    regAddr=0xD6 # ChipID
    byteOffset=0 # Level2Offset
    readCount=5  # 5바이트 
    regValueList=[0 for _ in range(readCount)]  #읽을 Register 값 개수만큼 리스트 할당 
    isOK = device.ReadReg(regAddr, byteOffset, readCount, regValueList)  #레지스터 Read함수 
    if isOK != True :
        print("FAIL: ReadReg,  Check the connection between TMonitor and T4/T5 Device")
        quit()

    # ChipID 읽은 결과 출력
    for regValue in regValueList :
        print(hex(regValue))

if __name__ == "__main__" :
    
    #
    #  T5에 연결후,  ChipID를 읽고, 화면에 출력하는 예제 
    #
    print("----------------------------------------------")
    print("[anapass-python::Example] Read ChipID")
    print("----------------------------------------------")

    #TDevice 객체 생성, T5연결하는 Device객체이다.
    #device = TDevice(TDevice.Type.T5)
    device = TDevice(TDevice.Type.TESys)

    ReadChipID(device)

    #연결을 끊는다.     
    print("Disconnect from " + device.GetName() )
    device.Disonnect()

    print("End of Exam. Bye!!")
    print()







#
# Ana6705/FlashMipiToSpi
#
from Anapass import Register
import time

ModuleName="Anapass.ANA6705.FlashMipiToSpi"
FlashCapacity = 0x1000000   # 16MByte
FlashSectorMaxCount = FlashCapacity >> 12
CtrlRegCount = 5 
FlashReadChunkByteSize = 128
FlashWriteChunkByteSize = 128
FlashSectorByteSize = 4096

#Spi Command 
COMMAND_READ = 0
COMMAND_ERASE = 1
COMMAND_SECTOR_ERASE = 2
COMMAND_WRITE = 3
COMMAND_READ_STATUS1 = 4
COMMAND_READ_STATUS2 = 5
COMMAND_WRITE_STATUS1 = 6
COMMAND_WRITE_STATUS2 = 7

def ConvertSectorAddrToMemAddr(sectorAddr) :
    return (sectorAddr << 12) & 0x00FFF000;

def ConvertMemAddrToSectorAddr(memAddr) :
    return (memAddr >> 12)

def MakeCmdValue(command, op_start) :
    c = ((command<<1) & 0x0E) | (op_start&0x1)
    return c

def WaitDone(device, spiCmd) :
    sleepMSec = 0.03; #30ms
    expectedResponse = spiCmd & 0xFE
    byteOffset=4
    bRet = False
    while 1 :
        c = device.ReadReg1Byte(Register.ANA6705.SPICTL, byteOffset)
        if c == -1 :
            raise AssertionError(ModuleName+ "." + "WaitDone")
        elif (c & 0x01) == 0 :
            if c == expectedResponse :
                bRet = True
            else :
                raise AssertionError(ModuleName+ "." + "WaitDone" + ":  NotExpected Value ")
            break
        else :
            time.sleep(sleepMSec)
            if sleepMSec < 1 :
                sleepMSec += 0.03 # 이 루프를 반복할때마다 30ms씩 대기시간 증가 (최대 1초) 
            else  :
                raise AssertionError(ModuleName+ "." + "WaitDone" + ":  TimeOut, check device")

    return bRet

def ReadStatus(device, command) :
    regValue=[0, 0, 0, 0, 0]  
    spiCmd = MakeCmdValue(command, 1)
    regValue[4] = spiCmd
    byteOffset=0
    isOK = device.WriteReg(Register.ANA6705.SPICTL, byteOffset, len(regValue), regValue) 
    if isOK != True :
        raise AssertionError(ModuleName+ "." + "ReadStatus")

    WaitDone(device, spiCmd)

    byteOffset = 0
    status = device.ReadReg1Byte(Register.ANA6705.SPIDATA, byteOffset)
    if status == -1 :
        raise AssertionError(ModuleName+ "." + "ReadStatus")
    return status

def ReadStatus1(device) :
    return ReadStatus(device, COMMAND_READ_STATUS1)

def ReadStatus2(device) :
    return ReadStatus(device, COMMAND_READ_STATUS2)


def WriteStatus(device, command, status) :

    byteOffset = 0
    bRet = device.WriteReg1Byte(Register.ANA6705.SPIDATA, byteOffset, status)
    if bRet != True :
        raise AssertionError(ModuleName+ "." + "WriteStatus")

    regValue=[0, 0, 0, 0, 0]  
    spiCmd = MakeCmdValue(command, 1)
    regValue[4] = spiCmd
    byteOffset=0
    isOK = device.WriteReg(Register.ANA6705.SPICTL, byteOffset, len(regValue), regValue) 
    if isOK != True :
        raise AssertionError(ModuleName+ "." + "ReadStatus")

    WaitDone(device, spiCmd)

def WriteStatus1(device,status) :
    return WriteStatus(device, COMMAND_WRITE_STATUS1, status)

def WriteStatus2(device,status) :
    return WriteStatus(device, COMMAND_WRITE_STATUS2, status)


def Lock(device) :
    status2 = ReadStatus2(device)
    if (status2&0x02) != 0 :   #두번째 bit가 High이면 Unlock이다. 
        status2 &= ~0x02
        WriteStatus2(status2)
    return True

def Unlock(device) :
    status2 = ReadStatus2(device)
    if (status2&0x02) == 0 :   #두번째 bit가 Low이면 Lock상태이다.
        status2 = 0x02
        WriteStatus2(device, status2)
    return True

def IsLock(device) :
    status2 = ReadStatus2(device)
    if (status2&0x02) != 0 : 
        isLock = False   #두번째 bit가 High이면 Unlock이다. 
    else :
        isLock = True
    return isLock


def EraseSector(device, sectorAddr) :

    regValue=[0, 0, 0, 0, 0]  

    assert sectorAddr < FlashSectorMaxCount 
    memAddr = ConvertSectorAddrToMemAddr(sectorAddr)
    
    #Set Addr
    regValue[2] = memAddr & 0xFF;
    regValue[1] =(memAddr >>8) & 0xFF;
    regValue[0] = (memAddr >>16) & 0xFF;

    spiCmd = MakeCmdValue(COMMAND_SECTOR_ERASE, 1)
    regValue[4] = spiCmd
    
    byteOffset=0
    isOK = device.WriteReg(Register.ANA6705.SPICTL, byteOffset, len(regValue), regValue) 
    assert isOK

    bRet = WaitDone(device, spiCmd)
    
    return bRet

def EraseChip(device) :

    regValue=[0, 0, 0, 0, 0]  

    assert sectorAddr < FlashSectorMaxCount 
    memAddr = ConvertSectorAddrToMemAddr(sectorAddr)
    
    spiCmd = MakeCmdValue(COMMAND_CHIP_ERASE, 1)
    regValue[4] = spiCmd
    
    byteOffset=0
    isOK = device.WriteReg(Register.ANA6705.SPICTL, byteOffset, len(regValue), regValue) 
    assert isOK

    bRet = WaitDone(device, spiCmd)

    return bRet
    
def ReadDataFromHW(device, memAddr, readByteSize, dataList, dataListStartIdx=0) :

    regValue=[0, 0, 0, 0, 0]  

    assert (memAddr+readByteSize) < FlashCapacity
    assert readByteSize <= FlashReadChunkByteSize

    #Set Addr
    regValue[2] = memAddr & 0xFF
    regValue[1] =(memAddr >>8) & 0xFF
    regValue[0] = (memAddr >>16) & 0xFF

    #읽을 개수 설정 
    regValue[3] = readByteSize - 1

    #Set Cmd
    spiCmd = MakeCmdValue(COMMAND_READ, 1)
    regValue[4] = spiCmd;
    isOK = device.WriteReg(Register.ANA6705.SPICTL, 0 , len(regValue), regValue) 
    assert isOK 

    bRet = WaitDone(device, spiCmd)

    if bRet :
        isOK = device.ReadReg(Register.ANA6705.SPIDATA, 0, readByteSize, dataList, dataListStartIdx)
        assert isOK 

    return bRet


def WriteDataToHW(device, memAddr, writeByteSize, writeData, writeDataStartIdx) : 

    bRet = True
    regValue = [0,0,0,0,0]
    
    dataIdx = 0;

    assert ((memAddr + writeByteSize) <= FlashCapacity)
    assert (writeByteSize <= FlashWriteChunkByteSize)

    #//Step1. Write To Buffer
    isOK = device.WriteReg(Register.ANA6705.SPIDATA, 0 , writeByteSize, writeData, writeDataStartIdx) 
    assert isOK 

    #Set Addr
    regValue[2] = memAddr & 0xFF
    regValue[1] = (memAddr >> 8) & 0xFF
    regValue[0] = (memAddr >> 16) & 0xFF

    #Write할 개수 
    regValue[3] = writeByteSize - 1; #//SPI_LENG[0,7] SPI_LENG+1 Bytes

    #Set Cmd
    spiCmd = MakeCmdValue(COMMAND_WRITE, 1)
    regValue[4] = spiCmd
    isOK = device.WriteReg(Register.ANA6705.SPICTL, 0 , len(regValue), regValue) 
    assert isOK

    #//Step3. Wait
    bRet = WaitDone(device, spiCmd)
        
    return bRet;




from enum import Enum
import time
import platform
import sys
import enum
import ted
from abc import ABCMeta, abstractmethod
from Anapass.TDeviceBase  import *

#
# class TDevice
#
class TDeviceAndroid(TDeviceBase) :

    def __init__(this, deviceTypeValue):
        TDeviceBase.__init__(this, deviceTypeValue)

    def __del__(this): pass
        #print("TDevice::~TDevice")
        #TDeviceDestroy(this.__DeviceHandle)
            
    def SysSetServerIPAddr(this, serverIPAddr) :
        return True

    def Connect(this) :
        return True

    def Disonnect(this) :
        return True
    
    def SysSetBoardID(this, boardID) : 
        return ted.SysSetBoardID(boardID)

    def SetTcLocalSave(this, boardID, bFlag) : 
        return ted.SysSetTcLocalSave(boardID, bFlag)

    def NotifyPyStart(this, pyFileName) : 
        return ted.SysNotifyPyStart(pyFileName)

    def NotifyPyStop(this, pyFileName) : 
        return ted.SysNotifyPyStop(pyFileName)

    def SysGetDutIndexAllDeviceValue(this) : 
        return ted.SysGetDutIndexAllDeviceValue()

    def SysGetDutCount(this) : 
        return ted.SysGetDutCount()

    def SysDelay(this, delay) :
        return ted.SysDelayMS(delay)

    def SysGetTickCount64(this) : 
        return ted.SysGetTickCount64()

    def SysGetCurUtcTime(this) : 
        return ted.SysGetCurUtcTime()

    def SysGetUtcTimeKST(this, year, month, day, hour, min, sec) : 
        return ted.SysGetUtcTimeKST(year, month, day, hour, min, sec)

    def SysGetErrFlag(this) : 
        return ted.SysGetErrFlag()

    def SysMipiLock(this) : 
        return ted.SysMipiLock()

    def SysMipiUnlock(this) : 
        return ted.SysMipiUnlock()

    def SysMipiIsLock(this) : 
        return ted.SysMipiIsLock()


    def SendTxtCmd(this, cmd) :
        return False

    def SendTxtCmdReadResp(this, cmd, maxRespByteSize) :
        return False

    #private methond
    def SendCtrlCmd(this, cmd) :  
        return False
    
    def ReadReg(this, regAddr, byteOffset, readCount, regValueList, regValueListStartIdx=0) :
        regValueListTemp = ted.DD_DSIM_MipiReadReg(regAddr, byteOffset, readCount)
        for idx, regValueInt in enumerate(regValueListTemp) :
            regValueList[idx + regValueListStartIdx] = regValueInt
        return True

    def ReadReg1Byte(this, regAddr, byteOffset) :
        return ted.DD_DSIM_MipiReadReg1Byte(dutIdx, regAddr, byteOffset)

    def WriteReg(this, regAddr, byteOffset, writeCount, regValueList, writeDataStartIdx=0) :

        dutIdx = 0

        regValueListTmp=[0 for _ in range(writeCount)]  #���� Register �� ������ŭ ����Ʈ �Ҵ� 
        for i in range(writeCount) :
            regValueListTmp[i] = regValueList[i+wrietDataStartIdx]
         
        ted.DD_DSIM_MipiWriteReg(dutIdx, regAddr, byteOffset, regValueListTmp)

        return True

    def WriteReg1Byte(this, regAddr, byteOffset, regValue) :
        return ted.DD_DSIM_MipiWriteReg1Byte(dutIdx, regAddr, byteOffset, regValue)

    def WriteCtrlReg(this, regAddr) :
        return this.WriteReg1Byte(regAddr, 0, 1)

    #COMM_API Bool TedDD_DSIM_MipiReadReg(int dutIdx, int addr, int byteOffset, int readCount, unsigned char* buf, int bufMaxByteSize);
    def DD_DSIM_MipiReadReg(this, dutIdx, regAddr, byteOffset, readCount) : 
        return ted.DD_DSIM_MipiReadReg(dutIdx, regAddr, byteOffset, readCount)

    #COMM_API unsigned char TedDD_DSIM_MipiReadReg1Byte(int dutIdx, int addr, int byteOffset);
    def DD_DSIM_MipiReadReg1Byte(this, dutIdx, regAddr, byteOffset) : 
        return ted.DD_DSIM_MipiReadReg1Byte(dutIdx, regAddr, byteOffset)

    #COMM_API Bool TedDD_DSIM_MipiReadReg(int dutIdx, int addr, int byteOffset, int readCount, unsigned char* buf, int bufMaxByteSize);
    def DD_DSIM_MipiWriteReg(this, dutIdx, regAddr, byteOffset, regValueList) : 
        return ted.DD_DSIM_MipiWriteReg(dutIdx, regAddr, byteOffset, regValueList)

    #COMM_API Bool TedDD_DSIM_MipiWriteReg1Byte(int dutIdx, int addr, int byteOffset, unsigned char data);
    def DD_DSIM_MipiWriteReg1Byte(this, dutIdx, regAddr, byteOffset, regValue) : 
        return ted.DD_DSIM_MipiWriteReg1Byte(dutIdx, regAddr, byteOffset, regValue)

    #WREG0=0x39, [Addr], [regVal0], [regVal1].....
    #COMM_API Bool TedDD_DSIM_MipiWriteReg39(int dutIdx, int addr, int writeCount, unsigned char* buf);
    def DD_DSIM_MipiWriteReg39(this, dutIdx, regAddr, regValueList) :
        return ted.DD_DSIM_MipiWriteReg39(dutIdx, regAddr, regValueList)

    #WREG0=0x15, [Addr], [regVal]
    #COMM_API Bool TedDD_DSIM_MipiWriteReg15(int dutIdx, int addr, unsigned char value);
    def DD_DSIM_MipiWriteReg15(this, dutIdx, regAddr, regValue) :
        return ted.DD_DSIM_MipiWriteReg15(dutIdx, regAddr, regValue)

    #WREG0=0x05, [Addr]
    #COMM_API Bool TedDD_DSIM_MipiWriteReg05(int dutIdx, int addr);
    def DD_DSIM_MipiWriteReg05(this, dutIdx, regAddr) :
        return ted.DD_DSIM_MipiWriteReg05(dutIdx, regAddr)

    #WREG0=0x07, [value]   :   Compressd Mode Command
    #COMM_API Bool TedDD_DSIM_MipiWriteReg07(int dutIdx, int addr);
    def DD_DSIM_MipiWriteReg07(this, dutIdx, value) :
        return ted.DD_DSIM_MipiWriteReg07(dutIdx, value)

    def CatchPower(this, powerInfo) :

        ret = TDeviceCatchPowerInfo(this.__DeviceHandle,  this.__PowerStructData, 1000)

        result= struct.unpack(this.__PowerStructFmt, this.__PowerStructData)

        resIdx=0
        
        powerInfo.No = result[resIdx] 
        resIdx += 1

        for i in range(10) :
            powerInfo.Avail[i] = result[i+resIdx]
        resIdx += 10

        for i in range(10) :
            powerInfo.Value1[i] = result[i+resIdx]
        resIdx += 10

        for i in range(10) :
            powerInfo.Voltage[i] = result[i+resIdx]
        resIdx += 10

        for i in range(10) :
            powerInfo.Current[i] = result[i+resIdx]
        resIdx += 10

        for i in range(10) :
            powerInfo.Range1[i] = result[i+resIdx]
        resIdx += 10

        for i in range(10) :
            powerInfo.Range2[i] = result[i+resIdx]
        resIdx += 10

        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag
    

    def PatternConnect(this) : 
        return ted.PatternConnect()

    def PatternDisconnect(this) : 
        return ted.PatternDisconnect()

    def PatternSetCommand(this, ptrnCmd) :
        return ted.PatternSetCommand(ptrnCmd)
    
    def PatternPaint(this, r, g, b, a) : 
        return ted.PatternPaint(r, g, b, a)

    def PatternUpdateScreen(this) : 
        return ted.PatternUpdateScreen()

    def PatternDrawImage(this, imgFileName) : 
        return ted.PatternDrawImage(imgFileName)

    def PatternScreenVerify(this, r, g, b) : 
        return ted.PatternScreenVerify(r, g, b)

    def ANA670X_GetChipIDCount(this) : 
        return ted.ANA670X_GetChipIDCount()

    def ANA670X_GetChipID(this, dutIdx) : 
        return ted.ANA670X_GetChipID(dutIdx)

    def ANA670X_GetProductRevisionBytesCount(this) : 
        return ted.ANA670X_GetProductRevisionBytesCount()

    def ANA670X_GetProductRevisionBytes(this, dutIdx) : 
        return ted.ANA670X_GetProductRevisionBytes(dutIdx)

    def ANA670X_SetFrameRate(this, dutIdx, fr) :
        return ted.ANA670X_SetFrameRate(dutIdx, fr)

    def ANA670X_GetFrameRate(this, dutIdx) : 
        return ted.ANA670X_GetFrameRate(dutIdx)

    def DebugMessage(this, msg) :
        #print(msg)
        ret = TDeviceDebugMessage(this.__DeviceHandle, TString.ConvertToCTypeStrng(msg))
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    def DebugFuncEnter(this, funcName) :
        ret = TDeviceDebugFuncEnter(this.__DeviceHandle, TString.ConvertToCTypeStrng(funcName))
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    def DebugFuncLeave(this, funcName) :
        ret = TDeviceDebugFuncLeave(this.__DeviceHandle, TString.ConvertToCTypeStrng(funcName))
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    def DD_FB_blank(this, dutIdx, value) :
        return ted.DD_FB_blank(dutIdx, value)

    #COMM_API Bool TedDD_DSIM_manual_ctrl(int dutIdx, int value);
    def DD_DSIM_manual_ctrl(this, dutIdx, value) :
        return ted.DD_DSIM_manual_ctrl(dutIdx, value)

    #COMM_API Bool TedDD_DSIM_power_ctrl(int dutIdx, int value);
    def DD_DSIM_power_ctrl(this, dutIdx, value) :
        return ted.DD_DSIM_power_ctrl(dutIdx, value)

    #COMM_API Bool TedDD_DSIM_source_cal(int dutIdx, int value);
    def DD_DSIM_source_cal(this, dutIdx, value) :
        return ted.DD_DSIM_source_cal(dutIdx, value)

    def DD_DSIM_sleepin(this, dutIdx, value) : 
        return ted.DD_DSIM_sleepin(dutIdx, value)
    
    def DD_DSIM_sleepout(this, dutIdx, value) : 
        return ted.DD_DSIM_sleepout(dutIdx, value)
    
    def DD_DSIM_deep_standby(this, dutIdx, value) : 
        return ted.DD_DSIM_deep_standby(dutIdx, value)
    
    def DD_DSIM_displayon(this, dutIdx, value) : 
        return ted.DD_DSIM_displayon(dutIdx, value)
    
    def DD_DSIM_reset_ctrl(this, dutIdx, value) : 
        return ted.DD_DSIM_reset_ctrl(dutIdx, value)


    #//VLIN1_ADC
    #TDEVICE_API TED_BOOL TDeviceVlin1AdcSetSamples(TDEVICE_HDL hdl, int value);  
    def Vlin1AdcSetSamples(this, dutIdx, value) :
        return ted.vlin1_adc_set_samples(dutIdx, value)

    #DEVICE_API TED_BOOL TDeviceVlin1AdcSetInterval(TDEVICE_HDL hdl, int value); 
    def Vlin1AdcSetInterval(this, dutIdx, value) :
        return ted.vlin1_adc_set_interval(dutIdx, value)

    #TDEVICE_API TED_BOOL TDeviceVlin1AdcSetChannelOn(TDEVICE_HDL hdl, int chIdx);
    def Vlin1AdcSetChannelOn(this, dutIdx, value) :
        return ted.vlin1_adc_set_channel_on(dutIdx, value)

    #TDEVICE_API TED_BOOL TDeviceVlin1AdcSetChannelOff(TDEVICE_HDL hdl, int chIdx); 
    def Vlin1AdcSetChannelOff(this, dutIdx, value) :
        return ted.vlin1_adc_set_channel_off(dutIdx, value)

    #TDEVICE_API TED_BOOL TDeviceVlin1AdcSetMode(TDEVICE_HDL hdl, int value);  
    def Vlin1AdcSetMode(this, dutIdx, value) :
        return ted.vlin1_adc_set_mode(dutIdx, value)

    #TDEVICE_API TED_BOOL TDeviceVlin1AdcGetVoltage(TDEVICE_HDL hdl, int chIdx);  
    def Vlin1AdcGetVoltage(this, dutIdx, chIdx) :
        return ted.vlin1_adc_get_voltage(dutIdx, chIdx)

    #TDEVICE_API TED_BOOL TDeviceVlin1AdcGetVoltageVLIN1(TDEVICE_HDL hdl);  
    def Vlin1AdcGetVoltageVLIN1(this, dutIdx) :
        return ted.vlin1_adc_get_voltage_VLIN1(dutIdx)

    #TDEVICE_API TED_BOOL TDeviceVlin1AdcGetVoltageVBAT(TDEVICE_HDL hdl);  
    def Vlin1AdcGetVoltageVBAT(this, dutIdx) :
        return ted.vlin1_adc_get_voltage_VBAT(dutIdx)

    #TDEVICE_API TED_BOOL TDeviceVlin1AdcGetVoltageELVDD(TDEVICE_HDL hdl);  
    def Vlin1AdcGetVoltageELVDD(this, dutIdx) :
        return ted.vlin1_adc_get_voltage_ELVDD(dutIdx)

    #TDEVICE_API TED_BOOL TDeviceVlin1AdcGetCurrent(TDEVICE_HDL hdl, int chIdx);  
    def Vlin1AdcGetCurrent(this, dutIdx, chIdx) :
        return ted.vlin1_adc_get_current(dutIdx, chIdx)

    #TDEVICE_API TED_BOOL TDeviceVlin1AdcGetCurrentVLIN1(TDEVICE_HDL hdl); 
    def Vlin1AdcGetCurrentVLIN1(this, dutIdx) :
        return ted.vlin1_adc_get_current_VLIN1(dutIdx)

    #TDEVICE_API TED_BOOL TDeviceVlin1AdcGetCurrentVBAT(TDEVICE_HDL hdl); 
    def Vlin1AdcGetCurrentVBAT(this, dutIdx) :
        return ted.vlin1_adc_get_current_VBAT(dutIdx)

    #TDEVICE_API TED_BOOL TDeviceVlin1AdcGetCurrentELVDD(TDEVICE_HDL hdl); 
    def Vlin1AdcGetCurrentELVDD(this, dutIdx) :
        return ted.vlin1_adc_get_current_ELVDD(dutIdx)

    #//VCI_ADC
    #TDEVICE_API TED_BOOL TDeviceVciAdcSetSamples(TDEVICE_HDL hdl, int value); 
    def VciAdcSetSamples(this, dutIdx, value) :
        return ted.vci_adc_set_samples(dutIdx, value)

    #TDEVICE_API TED_BOOL TDeviceVciAdcSetInterval(TDEVICE_HDL hdl, int value);
    def VciAdcSetInterval(this, dutIdx, value) :
        return ted.vci_adc_set_interval(dutIdx, value)

    #TDEVICE_API TED_BOOL TDeviceVciAdcSetChannelOn(TDEVICE_HDL hdl, int chIdx);
    def VciAdcSetChannelOn(this, dutIdx, value) :
        return ted.vci_adc_set_channel_on(dutIdx, value)

    #TDEVICE_API TED_BOOL TDeviceVciAdcSetChannelOff(TDEVICE_HDL hdl, int chIdx);
    def VciAdcSetChannelOff(this, dutIdx, value) :
        return ted.vci_adc_set_channel_off(dutIdx, value)

    #TDEVICE_API TED_BOOL TDeviceVciAdcSetMode(TDEVICE_HDL hdl, int value);
    def VciAdcSetMode(this, dutIdx, value) :
        return ted.vci_adc_set_mode(dutIdx, value)

    #TDEVICE_API int TDeviceVciAdcGetVoltage(TDEVICE_HDL hdl, int chIdx);
    def VciAdcGetVoltage(this, dutIdx, chIdx) :
        return ted.vci_adc_get_voltage(dutIdx, chIdx)

    #TDEVICE_API int TDeviceVciAdcGetVoltageVCI(TDEVICE_HDL hdl);
    def VciAdcGetVoltageVCI(this, dutIdx) :
        return ted.vci_adc_get_voltage_VCI(dutIdx)

    #TDEVICE_API int TDeviceVciAdcGetVoltageVDDR(TDEVICE_HDL hdl);
    def VciAdcGetVoltageVDDR(this, dutIdx) :
        return ted.vci_adc_get_voltage_VDDR(dutIdx)

    #TDEVICE_API int TDeviceVciAdcGetVoltageVDDI(TDEVICE_HDL hdl);
    def VciAdcGetVoltageVDDI(this, dutIdx) :
        return ted.vci_adc_get_voltage_VDDI(dutIdx)

    #TDEVICE_API int TDeviceVciAdcGetCurrent(TDEVICE_HDL hdl, int chIdx);
    def VciAdcGetCurrent(this, dutIdx, chIdx) :
        return ted.vci_adc_get_current(dutIdx, chIdx)

    #DEVICE_API int TDeviceVciAdcGetCurrentVCI(TDEVICE_HDL hdl);
    def VciAdcGetCurrentVCI(this, dutIdx) :
        return ted.vci_adc_get_current_VCI(dutIdx)

    #TDEVICE_API int TDeviceVciAdcGetCurrentVDDR(TDEVICE_HDL hdl);
    def VciAdcGetCurrentVDDR(this, dutIdx) :
        return ted.vci_adc_get_current_VDDR(dutIdx)

    #TDEVICE_API int TDeviceVciAdcGetCurrentVDDI(TDEVICE_HDL hdl);
    def VciAdcGetCurrentVDDI(this, dutIdx) :
        return ted.vci_adc_get_current_VDDI(dutIdx)

    #//SDOUT ADC
    #define TED_SDOUTADC_MAX_CH_COUNT 16
    #TDEVICE_API int TDeviceSoutAdcSetDevConfig(TDEVICE_HDL hdl, int value);
    def SoutAdcSetDevConfig(this, dutIdx, value) :
        return ted.sout_adc_set_devconfig(dutIdx, value)

    #TDEVICE_API int TDeviceSoutAdcSetInConfig(TDEVICE_HDL hdl, int chIdx,  int value); 
    def SoutAdcSetInConfig(this, dutIdx, chIdx, value) :
        return ted.sout_adc_set_inconfig(dutIdx, chIdx, value)

    #COMM_API Bool TedAdcSoutSetRBSel(int dutIdx, int value)
    def AdcSoutSetRBSel(this, dutIdx, value) :
        return ted.AdcSoutSetRBSel(dutIdx, value)

    #COMM_API int TedAdcSoutGetRBSel(int dutIdx)
    def AdcSoutGetRBSel(this, dutIdx) :
        return ted.AdcSoutGetRBSel(dutIdx)

    def SoutAdcGetChannelCount(this, dutIdx) :
        return ted.sout_adc_get_channel_count(dutIdx)

    #TDEVICE_API int TDeviceSoutAdcGetVoltage(TDEVICE_HDL hdl, int chIdx);  
    def SoutAdcGetVoltage(this, dutIdx, chIdx) :
        return ted.sout_adc_get_voltage(dutIdx, chIdx)

    #TDEVICE_API TED_BOOL TDeviceSoutAdcGetAllVoltage(TDEVICE_HDL hdl, int* voltageArray);
    def SoutAdcGetAllVoltage(this, dutIdx) :
        return ted.sout_adc_get_all_voltage(dutIdx)

    #TDEVICE_API TED_BOOL TDeviceLdoAdcSetInConfig(TDEVICE_HDL hdl, int chIdx, int value);
    def LdoAdcSetInConfig(this, dutIdx, chIdx, value) :
        return  ted.ldo_adc_set_inconfig(dutIdx, chIdx, value)

    def LdoAdcGetChannelCount(this, dutIdx) :
        return ted.ldo_adc_get_channel_count(dutIdx)

    #TDEVICE_API int TDeviceLdoAdcGetVoltage(TDEVICE_HDL hdl, int chIdx);
    def LdoAdcGetVoltage(this, dutIdx, chIdx) :
        return  ted.ldo_adc_get_voltage(dutIdx, chIdx)

    #TDEVICE_API TED_BOOL TDeviceLdoAdcGetAllVoltage(TDEVICE_HDL hdl, int* voltageArray);
    def LdoAdcGetAllVoltage(this, dutIdx) :
        return ted.ldo_adc_get_all_voltage(dutIdx)

    #TDEVICE_API TED_BOOL TDeviceRegAdcSetInConfig(TDEVICE_HDL hdl, int chIdx, int value);
    def RegAdcSetInConfig(this, dutIdx, chIdx, value) :
        return ted.reg_adc_set_inconfig(dutIdx, chIdx, value)

    def RegAdcGetChannelCount(this, dutIdx) :
        return ted.reg_adc_get_channel_count(dutIdx)

    #TDEVICE_API int TDeviceRegAdcGetVoltage(TDEVICE_HDL hdl, int chIdx);
    def RegAdcGetVoltage(this, dutIdx, chIdx) :
        return ted.reg_adc_get_voltage(dutIdx, chIdx)

    #TDEVICE_API TED_BOOL TDeviceLdoAdcGetAllVoltage(TDEVICE_HDL hdl, int* voltageArray);
    def RegAdcGetAllVoltage(this, dutIdx) :
        return ted.reg_adc_get_all_voltage(dutIdx)

    #COMM_API Bool TedAgingSetCurJobInfo(int dutIdx, int jobID, int status, int scIdx, int scCount, const char* desc);
    def AgingSetCurJobInfo(this, dutIdx, jobID, status, scIdx, scCount, desc) :
        return ted.AgingSetCurJobInfo(dutIdx, jobID, status, scIdx, scCount, desc)

    #TDEVICE_API TED_BOOL TDeviceAgingSetCurScInfo(TDEVICE_HDL hdl, int scID, int status, int tcIdx, int tcCount);
    def AgingSetCurScInfo(this, dutIdx, scID, status, tcIdx, tcCount, desc) :
        return ted.AgingSetCurScInfo(dutIdx, scID, status, tcIdx, tcCount, desc)

    #TDEVICE_API TED_BOOL TDeviceAgingSetCurTcInfo(TDEVICE_HDL hdl, int tcID, int status, int tcStepIdx, int tcStepCount);
    def AgingSetCurTcInfo(this, dutIdx, tcID, status, tcStepIdx, tcStepCount, desc) :
        return ted.AgingSetCurTcInfo(dutIdx, tcID, status, tcStepIdx, tcStepCount, desc)

    #TDEVICE_API TED_BOOL TDeviceAgingSetCurTcStepInfo(TDEVICE_HDL hdl, int tcStepID, int status);
    def AgingSetCurTcStepInfo(this, dutIdx, tcStepID, status, desc) :
        return ted.AgingSetCurTcStepInfo(dutIdx, tcStepID, status, desc)

    #COMM_API Bool TedAgingMeasureADC(int dutIdx, /*OUT*/void* res);
    def AgingMeasureADC(this, dutIdx) :
        return ted.AgingMeasureADC(dutIdx)

    #COMM_API int TedAgingMeasureADCResultStructureByteSize();
    def AgingMeasureADCResultStructureByteSize(this) : 
        return ted.AgingMeasureADCResultStructureByteSize()


#
# class TFileTransfer
#
class TFileTransfer :

    class Type(enum.IntEnum) : 
        T5 = 0
        
    class ErrorType(enum.IntEnum) : 
        Success = 0,
        SendPacket=1,
        NoResp=2,
        FileOpen=3,
        StorageSize=4,
        CRC=5
    
    #TDEVICE_API TFILETRANSFER_HDL TFileTransferCreate(enum TFileTransferType type, TDEVICE_HDL deviceHandle);
    def __init__(this, type, device) :
        this.__TFileTransferHandle = TFileTransferCreate(type, device.Handle)
        this.__FileName = ""

    def __getattr__(this, attrName) :
        if attrName == 'LastErrorString' : 
            return this.GetLastErrorString()
        if attrName == 'FileName' : 
            return this.__FileName
        else :
            raise AttributeError(attrName)


    #TDEVICE_API TED_BOOL TFileTransferDestroy(TFILETRANSFER_HDL fileTransferHandle);

    #TDEVICE_API TED_BOOL TFileTransferStart(TFILETRANSFER_HDL fileTransferHandle, const char* fileName);
    def Start(this, fileName) : 
        this.__FileName = fileName
        bytesString = fileName.encode('euc-kr')
        #bytesString = fileName.encode('ascii')
        #bytesString = fileName.encode('utf-8')
        ret = TFileTransferStart(this.__TFileTransferHandle, bytesString)
        return ret

    #TDEVICE_API TED_BOOL TFileTransferStop(TFILETRANSFER_HDL fileTransferHandle);
    def Stop(this) : 
        ret = TFileTransferStop(this.__TFileTransferHandle)
        return ret

    #TDEVICE_API int TFileTransferGetFileByteSize(TFILETRANSFER_HDL fileTransferHandle);
    def GetFileByteSize(this) : 
        ret = TFileTransferGetFileByteSize(this.__TFileTransferHandle)
        return ret

    #TDEVICE_API int TFileTransferGetTransferByteSize(TFILETRANSFER_HDL fileTransferHandle);
    def GetTransferByteSize(this) : 
        ret = TFileTransferGetTransferByteSize(this.__TFileTransferHandle)
        return ret

    #TDEVICE_API TED_BOOL TFileTransferIsStart(TFILETRANSFER_HDL fileTransferHandle);
    def IsStart(this) : 
        ret = TFileTransferIsStart(this.__TFileTransferHandle)
        return ret

    #TDEVICE_API TED_BOOL TFileTransferIsDone(TFILETRANSFER_HDL fileTransferHandle);
    def IsDone(this) : 
        ret = TFileTransferIsDone(this.__TFileTransferHandle)
        return ret

    #TDEVICE_API TED_BOOL TFileTransferIsError(TFILETRANSFER_HDL fileTransferHandle);
    def IsError(this) : 
        ret = TFileTransferIsError(this.__TFileTransferHandle)
        return ret

    #TDEVICE_API enum TFileTransferError TFileTransferGetLastError(TFILETRANSFER_HDL fileTransferHandle);
    def GetLastError(this) : 
        ret = TFileTransferGetLastError(this.__TFileTransferHandle)
        return ret

    def GetLastErrorString(this) :
        err = this.GetLastError()
        if err == TFileTransfer.ErrorType.Success :
            return "Success"
        elif err == TFileTransfer.ErrorType.SendPacket :
            return "SendPacket Error"
        elif err == TFileTransfer.ErrorType.NoResp :
            return "NoResp Error"
        elif err == TFileTransfer.ErrorType.FileOpen :
            return "FileOpen Error"
        elif err == TFileTransfer.ErrorType.StorageSize :
            return "StorageSize Error"
        elif err == TFileTransfer.ErrorType.CRC :
            return "CRC Error"
        else :
            return "Unknown Error"
    
    